#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is the parent class for all objects having refs

"""


from libsignetsim.settings.Settings import Settings


class HasSBaseRef(object):

	def __init__(self, model):

		self.__model = model


		self.__idRef = None
		self.__portRef = None
		self.__unitRef = None
		self.__metaIdRef = None
		self.__SBaseRef = None



	def readSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		if sbml_object.isSetIdRef():
			self.__idRef = sbml_object.getIdRef()

		if sbml_object.isSetPortRef():
			self.__portRef = sbml_object.getPortRef()

		if sbml_object.isSetUnitRef():
			self.__unitRef = sbml_object.getUnitRef()

		if sbml_object.isSetMetaIdRef():
			self.__metaIdRef = sbml_object.getMetaIdRef()

		if sbml_object.isSetSBaseRef():
			self.__SBaseRef = HasSBaseRef(self.__model)
			self.__SBaseRef.readSbml(sbml_object.getSBaseRef(), sbml_level, sbml_version)


	def writeSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):


		if self.__portRef is not None:
			sbml_object.setPortRef(self.__portRef)

		if self.__idRef is not None:
			sbml_object.setIdRef(self.__idRef)

		if self.__unitRef is not None:
			sbml_object.setUnitRef(self.__unitRef)

		if self.__metaIdRef is not None:
			sbml_object.setMetaIdRef(self.__metaIdRef)

		if self.__SBaseRef is not None:
			sbml_sbaseref = sbml_object.createSBaseRef()
			self.__SBaseRef.writeSbml(sbml_sbaseref, sbml_level, sbml_version)

	# def getRef(self, t_model):
	#
	# 	t_sbase_ref = []
	# 	if self.hasIdRef():
	# 		if self.hasSBaseRef():
	# 			tt_model = t_model.listOfSubmodels.getBySbmlIdRef(self.getIdRef()).getModelObject()
	# 			t_sbase_ref = self.getSBaseRef().getRef(tt_model)
	# 			return [self.getIdRef()] + t_sbase_ref
	#
	# 		else:
	# 			return [t_model.listOfVariables.getBySbmlId(self.getIdRef()).getMetaId()] + t_sbase_ref
	#
	# 	elif self.hasMetaIdRef():
	# 		return [self.getMetaIdRef()] + t_sbase_ref

	def getRef(self, t_model):

		t_sbase_ref = []
		if self.hasIdRef():
			if self.hasSBaseRef():
				tt_model = t_model.listOfSubmodels.getBySbmlId(self.getIdRef()).getModelObject()
				t_sbase_ref = self.getSBaseRef().getRef(tt_model)
				return [self.getIdRef()] + t_sbase_ref

			else:
				return [t_model.listOfVariables.getBySbmlId(self.getIdRef()).getMetaId()] + t_sbase_ref

		elif self.hasMetaIdRef():
			return self.getMetaIdRef()

		elif self.hasPortRef():
			return self.getPortRef()



	def hasPortRef(self):
		return self.__portRef is not None

	def getPortRef(self):
		return self.__portRef

	def setPortRef(self, port_ref):
		self.__portRef = port_ref


	def hasIdRef(self):
		return self.__idRef is not None

	def getIdRef(self):
		return self.__idRef

	def setIdRef(self, id_ref, prefix=""):
		if id_ref is not None:
			self.__idRef = prefix + id_ref


	def hasUnitRef(self):
		return self.__unitRef is not None

	def getUnitRef(self):
		return self.__unitRef

	def setUnitRef(self, unit_ref, prefix=""):
		if unit_ref is not None:
			self.__unitRef = prefix + unit_ref


	def hasMetaIdRef(self):
		return self.__metaIdRef is not None

	def getMetaIdRef(self):
		return self.__metaIdRef

	def setMetaIdRef(self, meta_id_ref, prefix=""):
		if meta_id_ref is not None:
			self.__metaIdRef = prefix + meta_id_ref


	def hasSBaseRef(self):
		return self.__SBaseRef is not None

	def getSBaseRef(self):
		return self.__SBaseRef

	def setSBaseRef(self, sbase_ref, prefix=""):
		if sbase_ref is not None:
			self.__SBaseRef = prefix + sbase_ref



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject

from libsignetsim.model.sbml.UnitDefinition import UnitDefinition
from libsignetsim.settings.Settings import Settings
from copy import deepcopy

class ListOfUnitDefinitions(ListOf, HasIds, SbmlObject):

	def __init__ (self, model=None):

		self.__model = model
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		SbmlObject.__init__(self, model)


	def readSbml(self, sbml_unitDefinitions,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		for unitDefinition in sbml_unitDefinitions:
			t_unitDefinition = UnitDefinition(self.__model, self.nextId())
			t_unitDefinition.readSbml(unitDefinition, sbml_level, sbml_version)
			ListOf.add(self, t_unitDefinition)

		SbmlObject.readSbml(self, sbml_unitDefinitions, sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		for unitDefinition in self:
			unitDefinition.writeSbml(sbml_model, sbml_level, sbml_version)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbml_model.getListOfUnitDefinitions(), sbml_level, sbml_version)


	def new(self):

		t_unitDefinition = UnitDefinition(self.__model, self.nextId())
		ListOf.add(self, t_unitDefinition)
		return t_unitDefinition


	def copy(self, obj, deletions=[], usids_subs={}):

		if obj not in deletions:

			SbmlObject.copy(self, obj)

			for unit_definition in obj:
				if unit_definition not in deletions:

					t_definition = UnitDefinition(self.__model, self.nextId())
					t_definition.copy(unit_definition, usids_subs=usids_subs)
					ListOf.add(self, t_definition)


	def getAmountUnit(self, unit, compartment_unit):

		new_unit = deepcopy(unit)
		new_comp_unit = deepcopy(compartment_unit)
		new_comp_unit.listOfUnits[0].exponent *= -1

		new_unit_list = []
		for t_units in new_unit.listOfUnits:
			if not t_units.isEqual(new_comp_unit.listOfUnits[0]):
				new_unit_list.append(t_units)

		new_unit.listOfUnits = new_unit_list


		if not self.containsUnits(new_unit):
			ListOf.add(self, new_unit)
			return new_unit
		else:

			return self.getExistingUnits(new_unit)
		# return new_unit


	def getConcentrationUnit(self, unit, compartment_unit):

		new_unit = deepcopy(unit)
		new_comp_unit = deepcopy(compartment_unit)
		new_comp_unit.listOfUnits[0].exponent = new_comp_unit.listOfUnits[0].exponent * -1
		new_unit.listOfUnits = new_unit.listOfUnits + new_comp_unit.listOfUnits

		if not self.containsUnits(new_unit):
			ListOf.add(self, new_unit)
			return new_unit
		else:
			# We return the one already in the list, not the computed one
			return self.getExistingUnits(new_unit)


	def getExistingUnits(self, units):

		for t_unit in self:
			if units.isEqual(t_unit):
				return t_unit


	def containsUnits(self, units):

		for t_unit in self:
			if units.isEqual(t_unit):
				return True

		return False

	def isUnitIdAvailable(self, unit_id):
		return unit_id not in HasIds.sbmlIds(self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.HasMetaIds import HasMetaIds

class HasIds(HasMetaIds):
	""" Parent class for all the ListOf containers in a sbml model """

	def __init__ (self, model=None):

		self.__model = model
		HasMetaIds.__init__(self, model)



	def sbmlIds(self):
		""" Return a set of import ids of the sbml objects """
		return [obj.getSbmlId() for obj in self]

	def getBySbmlId(self, sbml_id, pos=0):
		""" Find sbml objects by their import Id """

		res = []
		for obj in self:
			if obj.getSbmlId() == sbml_id:
				res.append(obj)

		if len(res) > 0:
			return res[pos]


	def containsSbmlId(self, sbml_id):
		""" Test if an sbml id is in the list """

		res = False
		for obj in self:
			if sbml_id == obj.getSbmlId():
				res = True

		return res



	def names(self):
		""" Return set of names of the sbml objects """
		return [obj.getName() for obj in self]

	def getByName(self, name, pos=0):
		""" Find sbml objects by their name """

		res = []
		for obj in self:
			if obj.getName() == name:
				res.append(obj)

		if len(res) > 0:
			return res[pos]



	def containsName(self, name):
		""" Test if a name is in the list """

		res = False
		for obj in self:
			if name == obj.getName():
				res = True

		return res



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.Curve import Curve
from libsignetsim.settings.Settings import Settings

class ListOfCurves(ListOf):

	def __init__(self, document, plot):

		ListOf.__init__(self, document)

		self.__document = document
		self.__plot = plot
		self.__curvesCounter = 0

	def new(self, curve_id=None):
		t_curve_id = curve_id
		if t_curve_id is None:
			t_curve_id = "%s_curve_%d" % (self.__plot.getId(), self.__curvesCounter)

		curve = Curve(self.__document)
		curve.setId(t_curve_id)
		ListOf.append(self, curve)
		self.__curvesCounter += 1
		return curve

	def createCurve(self, curve_id=None):
		return self.new(curve_id)

	def readSedml(self, list_of_curves, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_curves, level, version)

		for t_curve in list_of_curves:
			curve = Curve(self.__document)
			curve.readSedml(t_curve, level, version)
			ListOf.append(self, curve)
			self.__curvesCounter += 1

	def writeSedml(self, list_of_curves, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_curves, level, version)

		for t_curve in self:
			curve = list_of_curves.createCurve()
			t_curve.writeSedml(curve, level, version)

	def getDataToGenerate(self):

		data = []
		for curve in self:
			data += curve.getDataToGenerate()

		return list(set(data))

	def getXAxisTitle(self):

		titles = []
		for curve in self:
			titles.append(curve.getXAxisTitle())

		if len(set(titles)) == 1:
			return titles[0]

	def getYAxisTitle(self):

		titles = []
		for curve in self:
			titles.append(curve.getYAxisTitle())

		if len(set(titles)) == 1:
			return titles[0]


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file the CWriterOptimization class definition, which writes
	the common code for the different optimizations.

"""


from libsignetsim.settings.Settings import Settings
from libsignetsim.optimization.OptimizationException import OptimizationCompilationException
from shutil import copytree, copyfile
from os.path import join, isfile
from os import mkdir
from glob import glob
from random import randrange


class CWriterOptimization(object):

	def __init__(self, workingModel, parameters,
				p_lambda=Settings.defaultPlsaLambda,
				p_criterion=Settings.defaultPlsaCriterion,
				p_precision=Settings.defaultPlsaPrecision,
				p_initial_temperature=Settings.defaultPlsaInitialTemperature,
				p_gain=Settings.defaultPlsaGainForJumpSizeControl,
				p_interval=Settings.defaultPlsaInterval,
				p_mix=Settings.defaultPlsaMixInterval,
				p_initial_moves=Settings.defaultPlsaInitialMoves,
				p_tau=Settings.defaultPlsaTau,
				p_freeze_count=Settings.defaultPlsaFreezeCount,
	):

		self.workingModel = workingModel
		self.parameters = parameters
		self.pLambda = p_lambda
		self.pCriterion = p_criterion
		self.pPrecision = p_precision
		self.pInitialTemp = p_initial_temperature
		self.pGain = p_gain
		self.pInterval = p_interval
		self.pMix = p_mix
		self.pInitialMoves = p_initial_moves
		self.pTau = p_tau
		self.pFreezeCount = p_freeze_count

	def writeOptimizationFiles(self, nb_procs):

		mkdir(join(self.getTempDirectory(), "src"))
		mkdir(join(self.getTempDirectory(), "lib"))

		# First the code
		mkdir(join(self.getTempDirectory(), "src/integrate"))
		copyfile(join(Settings.basePath, "lib/integrate/src/integrate.h"), join(self.getTempDirectory(), "src/integrate/integrate.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/models.h"), join(self.getTempDirectory(), "src/integrate/models.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/datas.h"), join(self.getTempDirectory(), "src/integrate/datas.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/realtype_math.h"), join(self.getTempDirectory(), "src/integrate/realtype_math.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/types.h"), join(self.getTempDirectory(), "src/integrate/types.h"))

		mkdir(join(self.getTempDirectory(), "src/plsa"))
		copyfile(join(Settings.basePath, "lib/plsa/src/sa.h"), join(self.getTempDirectory(), "src/plsa/sa.h"))
		copyfile(join(Settings.basePath, "lib/plsa/src/config.h"), join(self.getTempDirectory(), "src/plsa/config.h"))
		copyfile(join(Settings.basePath, "lib/plsa/src/types.h"), join(self.getTempDirectory(), "src/plsa/types.h"))

		copyfile(join(Settings.basePath, "lib/scoreFunctions.h"), join(self.getTempDirectory(), "src/scoreFunctions.h"))
		copyfile(join(Settings.basePath, "lib/scoreFunctions.c"), join(self.getTempDirectory(), "src/scoreFunctions.c"))

		# Then the shared
		if not isfile(join(Settings.basePath, "lib/integrate/integrate.so")):
			integrate_filename = glob(join(Settings.basePath, "lib", "integrate", "integrate*.so"))
			if len(integrate_filename) > 0:
				copyfile(integrate_filename[0],
						 join(self.getTempDirectory(), "lib/integrate.so"))
			else:
				raise OptimizationCompilationException("Could not find the numerical integration library. Please reinstall libSigNetSim")
		else:
			copyfile(join(Settings.basePath, "lib/integrate/integrate.so"), join(self.getTempDirectory(), "lib/integrate.so"))

		if not isfile(join(Settings.basePath, "lib/plsa/libplsa-serial.so")):
			plsa_serial_filename = glob(join(Settings.basePath, "lib", "plsa", "libplsa-serial*.so"))
			if len(plsa_serial_filename) > 0:
				copyfile(plsa_serial_filename[0],
						 join(self.getTempDirectory(), "lib/libplsa-serial.so"))
			else:
				raise OptimizationCompilationException(
					"Could not find the numerical integration library. Please reinstall libSigNetSim")
		else:
			copyfile(join(Settings.basePath, "lib/plsa/libplsa-serial.so"), join(self.getTempDirectory(), "lib/libplsa-serial.so"))

		if not isfile(join(Settings.basePath, "lib/plsa/libplsa-parallel.so")):
			plsa_parallel_filename = glob(join(Settings.basePath, "lib", "plsa", "libplsa-parallel*.so"))
			if len(plsa_parallel_filename) > 0:
				copyfile(plsa_parallel_filename[0],
						 join(self.getTempDirectory(), "lib/libplsa-parallel.so"))
			else:
				raise OptimizationCompilationException(
					"Could not find the numerical integration library. Please reinstall libSigNetSim")
		else:
			copyfile(join(Settings.basePath, "lib/plsa/libplsa-parallel.so"), join(self.getTempDirectory(), "lib/libplsa-parallel.so"))

		copyfile(join(Settings.basePath, "lib/templates/data_optimization/Makefile"), join(self.getTempDirectory(), "Makefile") )
		copyfile(join(Settings.basePath, "lib/templates/data_optimization/main.c"), join(self.getTempDirectory(), "src/main.c") )

	def writeOptimizationFilesHeaders(self, f_c, f_h):

		f_c.write("#include <stdlib.h>\n")
		f_c.write("#include <stdio.h>\n")
		f_c.write("#include \"optim.h\"\n")

		f_h.write("#include \"plsa/sa.h\"\n")
		f_h.write("#include \"plsa/config.h\"\n")
		f_h.write("#include \"integrate/models.h\"\n")
		f_h.write("#include \"scoreFunctions.h\"\n")

	def writeOptimizationGlobals(self, f_c, f_h):

		f_c.write("PArrPtr * my_plist;\n")

	def writeOptimizationGlobalMethods(self, f_c, f_h):

		f_h.write("PArrPtr * getOptimParameters(void);\n")
		f_c.write("PArrPtr * getOptimParameters(void)\n")
		f_c.write("{\n\treturn my_plist;\n}\n\n")

	def writeOptimizationParameters(self, f_c, f_h):

		f_h.write("void init_params(ModelDefinition * model);\n")
		f_c.write("void init_params(ModelDefinition * model)\n{\n")

		f_c.write("\tmy_plist = InitPLSAParameters(%d);\n" % len(self.parameters))

		for (i_param, (param, value, lb, ub, precision)) in enumerate(self.parameters):
			f_c.write("\tmy_plist->array[%d] = (ParamList) {&(model->constant_variables[%d].value), %g, (Range) {%g, %g}, %g, \"%s\"};\n" % (i_param, param.ind, value, lb, ub, precision, param.getXPath()))

		f_c.write("}")

	def writeOptimizationSettings(self, f_c, f_h):

		f_h.write("void init_settings(SAType * settings);\n")
		f_c.write("void init_settings(SAType * settings)\n{\n")

		f_c.write("\tsettings->seed = %g;\n" % self.randomPlsaSeed())
		f_c.write("\tsettings->initial_temp = %g;\n" % self.pInitialTemp)
		f_c.write("\tsettings->gain_for_jump_size_control = %g;\n" % self.pGain)
		f_c.write("\tsettings->interval = %g;\n" % self.pInterval)
		f_c.write("\tsettings->initial_moves = %g;\n" % self.pInitialMoves)
		f_c.write("\tsettings->tau = %g;\n" % self.pTau)
		f_c.write("#ifdef MPI\n")
		f_c.write("\tsettings->mix_interval = %g;\n" % self.pMix)
		f_c.write("#endif\n")
		f_c.write("\tsettings->lambda = %g;\n" % self.pLambda)
		f_c.write("\tsettings->criterion = %g;\n" % self.pCriterion)
		f_c.write("\tsettings->param_precision = %g;\n" % self.pPrecision)
		f_c.write("\tsettings->freeze_count = %d;\n" % self.pFreezeCount)
		f_c.write("}")


	def randomPlsaSeed(self):
		return randrange(-2147483647, +2147483647)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.container.ListOf import ListOf
from libsignetsim.numl.ResultComponent import ResultComponent

from libsignetsim.settings.Settings import Settings

class ListOfResultComponents(ListOf, list):

	def __init__(self, document):

		ListOf.__init__(self, document)
		self.__document = document
		self.__resultComponentCounter = 0

	def createResultComponent(self):

		result_component = ResultComponent(self.__document)
		result_component.setId("result_component_%d" % self.__resultComponentCounter)
		ListOf.append(self, result_component)
		self.__resultComponentCounter += 1
		return result_component

	def readNuML(self, list_of_result_component, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		ListOf.readNuML(self, list_of_result_component, level, version)

		for result_component in list_of_result_component:
			t_result_component = ResultComponent(self.__document)
			t_result_component.readNuML(result_component, level, version)
			ListOf.append(self, t_result_component)

	def writeNuML(self, numl_document, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		ListOf.writeNuML(self, numl_document.getResultComponents(), level, version)

		for result_component in self:
			t_result_component = numl_document.createResultComponent()
			result_component.writeNuML(t_result_component, level, version)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.model.sbml.SbmlModelCreator import SbmlModelCreator
from libsignetsim.settings.Settings import Settings

from libsbml import RDFAnnotationParser, ModelHistory, ModelCreator, Date
import libsbml
class SbmlModelHistory(object):
	""" Sbml model class """


	def __init__ (self, model):
		""" Constructor of model class """

		self.__model = []
		self.__listOfCreators = []
		self.__listOfModifiedDates = []
		self.__createdDate = None

	def addCreator(self, family_name, given_name, email, organisation=None, name=None):

		t_creator = SbmlModelCreator()
		t_creator.new(family_name, given_name, email, organisation, name)
		self.__listOfCreators.append(t_creator)

	def readSbml(self, model, level=Settings.defaultSbmlLevel, version=Settings.defaultSbmlVersion):

		if model.isSetModelHistory():
			history = model.getModelHistory()

			if history.isSetCreatedDate():
				self.__createdDate = history.getCreatedDate().getDateAsString()

			for creator in history.getListCreators():
				t_creator = SbmlModelCreator()
				t_creator.readSbml(creator, level, version)
				self.__listOfCreators.append(t_creator)

			for modified_date in history.getListModifiedDates():
				self.__listOfModifiedDates.append(modified_date.getDateAsString())
		# else:
		# 	print "fuck you"
	def writeSbml(self, model, level=Settings.defaultSbmlLevel, version=Settings.defaultSbmlVersion):

		if len(self.__listOfCreators) > 0 and len(self.__listOfModifiedDates) > 0 and self.__createdDate is not None:
			history = ModelHistory()

			for creator in self.__listOfCreators:
				t_creator = ModelCreator()
				creator.writeSbml(t_creator, level, version)
				history.addCreator(t_creator)

			if self.__createdDate is not None:
				t_date = Date()
				t_date.setDateAsString(self.__createdDate)
				history.setCreatedDate(t_date)

			for modifiedDate in self.__listOfModifiedDates:
				t_date = Date()
				t_date.setDateAsString(modifiedDate)
				history.addModifiedDate(t_date)

			model.setModelHistory(history)
			# print model.getAnnotation().getNumChildren()
			# print model.getAnnotation().getChild(0).getChild(0).toString()
			# print model.getAnnotation().getChild(0).getChild(0).getAttrValue(0)


	def getListOfCreators(self):
		return self.__listOfCreators

	def getListOfCreatorsEmails(self):
		return [creator.getEmail() for creator in self.__listOfCreators]

	def getDateCreated(self):
		return self.__createdDate

	def getListOfDateModified(self):
		return self.__listOfModifiedDates

	def __str__(self):
		if len(self.__listOfCreators) > 0 and len(self.__listOfModifiedDates) > 0 and self.__createdDate is not None:

			res = ">Creators : \n"
			for creator in self.__listOfCreators:
				res += ">> %s\n" % str(creator)

			res += "\n> Created : %s\n" % self.__createdDate
			res += "> Modified : \n"
			for modified in self.__listOfModifiedDates:
				res += ">> %s\n" % modified

		return res

	def createCreator(self):
		t_creator = SbmlModelCreator()
		self.__listOfCreators.append(t_creator)
		return t_creator

	def setDateCreated(self, date):
		self.__createdDate = date.getDateAsString()

	def addModifiedDate(self, date):
		self.__listOfModifiedDates.append(date.getDateAsString())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the computation of equilibrium curves

"""

from libsignetsim import SbmlDocument, EquilibriumPointCurve
from unittest import TestCase
from os.path import join, dirname
from os import getcwd

class TestContinuationDas(TestCase):
	""" Tests high level functions """

	def testAllReductions(self):

		vars = ['ras_gtp', 'ras_gdp', 'sos', 'sos_ras_gtp', 'sos_ras_gdp']
		tests = [
			(0, 1), (0, 2), (0, 3), (0, 4),
			(1, 2), (1, 3), (1, 4),
			(2, 3), (2, 4),
			(3, 4)
		]

		results = []

		testfiles_path = join(join(getcwd(), dirname(__file__)), "files")

		for test in tests:

			doc = SbmlDocument()
			doc.readSbmlFromFile(join(testfiles_path, "ras_das_full.xml"))
			doc.model.build(reduce=False)
			doc.model.listOfConservationLaws.build()

			vars_to_reduce = set(list(test))
			ind_to_keep = set(range(5)).difference(vars_to_reduce)
			vars_to_keep = [vars[ind] for ind in ind_to_keep]

			t_ep_curve = EquilibriumPointCurve(doc.model)
			t_ep_curve.setParameter(doc.model.listOfVariables.getBySbmlId("total_1"))
			t_ep_curve.setRange(0, 10000)
			t_ep_curve.setDs(1)
			t_ep_curve.setMaxSteps(5000)
			t_ep_curve.build(vars_to_keep=vars_to_keep)
			t_ep_curve.run()

			if t_ep_curve.getCurves() != None:
				x, ys = t_ep_curve.getCurves()
				results.append(len(x))

				if len(x) > 1:
					if x[len(x)-1] > 0:
						print("Model with %s : Success" % str(vars_to_keep))
					else:
						print("Model with %s : Went backward" % str(vars_to_keep))
				else:
					print("Model with %s : Failed" % str(vars_to_keep))
			else:
				print("Model with %s : Failed" % str(vars_to_keep))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.settings.Settings import Settings
from sympy import pretty
from time import time


class MathStoichiometryMatrix(object):
	""" Sbml model class """

	def __init__(self, model):
		""" Constructor of model class """

		self.__model = model
		self.stoichiometryMatrix = None

	def __str__(self):
		return pretty(self.stoichiometryMatrix)

	def pprint(self):
		print(pretty(self.stoichiometryMatrix))

	def build(self, including_fast_reactions=True, including_slow_reactions=True, include_variable_stoichiometry=False):

		t0 = time()

		subs = {}
		for var, value in list(self.__model.listOfInitialConditions.items()):
			subs.update({var: value.getInternalMathFormula()})

		matrix = None
		for i, reaction in enumerate(self.__model.listOfReactions):

			reaction_matrix = reaction.getStoichiometryMatrix(subs,
					including_fast_reactions=including_fast_reactions,
					including_slow_reactions=including_slow_reactions,
					include_variable_stoichiometry=include_variable_stoichiometry
			)

			if matrix is None:
				matrix = reaction_matrix
			else:
				matrix = matrix.row_join(reaction_matrix)

		self.stoichiometryMatrix = matrix

		if Settings.verboseTiming >= 2:
			print("reaction matrix built in %.2gs" % (time()-t0))

	def getStoichiometryMatrix(self, including_fast_reactions=True, including_slow_reactions=True, include_variable_stoichiometry=False):

		if self.stoichiometryMatrix is None:
			self.build(including_fast_reactions, including_slow_reactions, include_variable_stoichiometry)
		return self.stoichiometryMatrix



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase

from libsignetsim.sedml.container.ListOfModels import ListOfModels
from libsignetsim.sedml.container.ListOfSimulations import ListOfSimulations
from libsignetsim.sedml.container.ListOfTasks import ListOfTasks
from libsignetsim.sedml.container.ListOfDataGenerators import ListOfDataGenerators
from libsignetsim.sedml.container.ListOfOutputs import ListOfOutputs
from libsignetsim.sedml.container.ListOfIds import ListOfIds

from libsignetsim.sedml.SedmlException import SedmlFileNotFound

from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import readSedMLFromFile, writeSedMLToFile, SedDocument, writeSedMLToString
from six.moves import reload_module
reload_module(libsbml)

from os.path import dirname, basename, exists

class SedmlDocument(SedBase):
	""" Sedml document class """

	def __init__ (self):
		""" Constructor of model class """

		SedBase.__init__(self, self)

		self.path = None
		self.filename = None
		self.level = Settings.defaultSedmlLevel
		self.version = Settings.defaultSedmlVersion

		self.listOfModels = ListOfModels(self)
		self.listOfSimulations = ListOfSimulations(self)
		self.listOfTasks = ListOfTasks(self)
		self.listOfDataGenerators = ListOfDataGenerators(self)
		self.listOfOutputs = ListOfOutputs(self)

		self.listOfIds = ListOfIds(self)
		self.executionDuration = 0

	def readSedmlFromFile(self, filename):

		if not exists(filename):
			raise SedmlFileNotFound("SED-ML file %s not found" % filename)

		document = readSedMLFromFile(str(filename))

		self.path = dirname(filename)
		self.filename = basename(filename)
		self.readSedml(document)

	def readSedml(self, document):

		self.level = document.getLevel()
		self.version = document.getVersion()

		SedBase.readSedml(self, document, self.level, self.version)

		self.listOfModels.readSedml(document.getListOfModels(), self.level, self.version)
		self.listOfSimulations.readSedml(document.getListOfSimulations(), self.level, self.version)
		self.listOfTasks.readSedml(document.getListOfTasks(), self.level, self.version)
		self.listOfDataGenerators.readSedml(document.getListOfDataGenerators(), self.level, self.version)
		self.listOfOutputs.readSedml(document.getListOfOutputs(), self.level, self.version)

	def writeSedml(self, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		document = SedDocument()

		document.setLevel(self.level)
		document.setVersion(self.version)

		SedBase.writeSedml(self, document, self.level, self.version)

		self.listOfModels.writeSedml(document.getListOfModels(), self.level, self.version)
		self.listOfSimulations.writeSedml(document.getListOfSimulations(), self.level, self.version)
		self.listOfTasks.writeSedml(document.getListOfTasks(), self.level, self.version)
		self.listOfDataGenerators.writeSedml(document.getListOfDataGenerators(), self.level, self.version)
		self.listOfOutputs.writeSedml(document.getListOfOutputs(), self.level, self.version)
		return document

	def writeSedmlToFile(self, filename,
							level=Settings.defaultSedmlLevel,
							version=Settings.defaultSedmlVersion,
							write_sbml_dependencies=False):

		self.path = dirname(filename)
		self.filename = basename(filename)

		self.listOfModels.makeRelativePaths(self.path)

		if write_sbml_dependencies:
			self.listOfModels.writeSbmlModelsToPath(dirname(filename))

		document = self.writeSedml(level, version)
		writeSedMLToFile(document, str(filename))

	def writeSedmlToString(self,
							level=Settings.defaultSedmlLevel,
							version=Settings.defaultSedmlVersion,
		):

		document = self.writeSedml(level, version)

		return writeSedMLToString(document)


	def run(self):

		data_to_generate = self.listOfOutputs.getDataToGenerate()
		tasks_to_run = self.listOfDataGenerators.getTasksToRun(data_to_generate)

		for task in tasks_to_run:
			task.run()
			self.executionDuration += task.getDuration()

		self.listOfDataGenerators.build()
		# self.listOfOutputs.build()

	def showFigures(self):

		self.listOfOutputs.showFigures()


	def getSbmlDependencies(self):

		deps = []
		for model in self.listOfModels:
			if model.getSource().isLocal():
				deps.append(model.getSource().getFilename())

		return deps



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.EventAssignedVariable import EventAssignedVariable
from libsignetsim.model.sbml.InitiallyAssignedVariable import InitiallyAssignedVariable
from libsignetsim.model.sbml.RuledVariable import RuledVariable
from libsignetsim.model.Variable import Variable
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasUnits import HasUnits
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.settings.Settings import Settings


class Compartment(Variable, SbmlObject, InitiallyAssignedVariable,
						RuledVariable, EventAssignedVariable, HasUnits, HasParentObj):
	""" Compartment definition """


	def __init__ (self, model, parent_obj, obj_id):


		self.__model = model
		self.objId = obj_id

		SbmlObject.__init__(self, model)
		Variable.__init__(self, model, Variable.COMPARTMENT)
		InitiallyAssignedVariable.__init__(self, model)
		RuledVariable.__init__(self, model)
		EventAssignedVariable.__init__(self, model)
		HasUnits.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)
		self.spatialDimensions = None


	def new(self, name=None, sbml_id=None, value=1, constant=True, unit=None):
		""" Creates new compartment with default options """

		if sbml_id is None:
			sbml_id = name
		Variable.new(self, sbml_id, Variable.COMPARTMENT)
		SbmlObject.new(self, name)
		HasUnits.new(self, unit)

		self.setValue(value)
		self.constant = constant
		self.setUnits(self.__model.getCompartmentUnits())


	def copy(self, compartment, sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factors=None):

		SbmlObject.copy(self, compartment)
		HasUnits.copy(self, compartment, usids_subs=usids_subs)
		Variable.copy(self, compartment, sids_subs=sids_subs, symbols_subs=symbols_subs, conversion_factor=conversion_factors)

		self.spatialDimensions = compartment.spatialDimensions




	def readSbml(self, sbml_compartment,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		""" Reads compartment from a sbml file """

		SbmlObject.readSbml(self, sbml_compartment, sbml_level, sbml_version)
		Variable.readSbml(self, sbml_compartment, sbml_level, sbml_version)
		HasUnits.readSbml(self, sbml_compartment, sbml_level, sbml_version)


		# spatialDimensions
		if sbml_level >= 2:
			if sbml_compartment.isSetSpatialDimensions():
				if sbml_level == 2:
					self.spatialDimensions = sbml_compartment.getSpatialDimensions()
				else:
					self.spatialDimensions = sbml_compartment.getSpatialDimensionsAsDouble()
			elif sbml_level in [1, 2]:
				self.spatialDimensions = 3

			if self.spatialDimensions == 0:
				self.setValue(1)



	def readSbmlVariable(self, sbml_compartment,
							sbml_level=Settings.defaultSbmlLevel,
							sbml_version=Settings.defaultSbmlVersion):

		""" Reads the variable information from a sbml file """
		""" I'm doubting the purpose of isInitialized here... """

		# variable id
		self.symbol.readSbml(sbml_compartment.getId(), sbml_level, sbml_version)

		# Size/Volume
		if sbml_level >= 2:
			if sbml_compartment.isSetSize():
				self.isInitialized = True
				self.value.readSbml(sbml_compartment.getSize(),
										sbml_level, sbml_version)

		else:
			if sbml_compartment.isSetVolume():
				self.isInitialized = True
				self.value.readSbml(sbml_compartment.getVolume(),
									sbml_level, sbml_version)


		# constant
		if sbml_compartment.isSetConstant():
			self.constant = sbml_compartment.getConstant()
		elif sbml_level == 2:
			self.constant = True
		elif sbml_level == 1:
			self.constant = False



	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		""" Writes compartment to a sbml file """

		sbml_compartment = sbml_model.createCompartment()

		SbmlObject.writeSbml(self, sbml_compartment, sbml_level, sbml_version)
		Variable.writeSbml(self, sbml_compartment, sbml_level, sbml_version)
		HasUnits.writeSbml(self, sbml_compartment, sbml_level, sbml_version)

		# SpatialDimensions
		if sbml_level >= 2:
			if self.spatialDimensions is not None:
				if sbml_level == 2 and self.spatialDimensions == 3:
					pass
				else:
					sbml_compartment.setSpatialDimensions(self.spatialDimensions)



	def writeSbmlVariable(self, sbml_compartment,
							sbml_level=Settings.defaultSbmlLevel,
							sbml_version=Settings.defaultSbmlVersion):

		""" Writes the variable information to a sbml file """

		if self.constant is not None:
			if not (sbml_level == 2 and self.constant is True
				or sbml_level == 1 and self.constant is False):
				sbml_compartment.setConstant(self.constant)

		# Size/Volume
		if self.isInitialized and self.spatialDimensions != 0:
			t_value = self.getSbmlValue(sbml_level, sbml_version).getValue()
			if sbml_level >= 2:
				sbml_compartment.setSize(t_value)
			else:
				sbml_compartment.setVolume(t_value)



	def getSize(self):
		return self.value.getPrettyPrintMathFormula()


	def getSizeMath(self):
		return self.value


	def setSize(self , value):

		if self.value is None:
			self.value = MathFormula(self.__model)

		self.value.setPrettyPrintMathFormula(value)


	def getXPath(self, attribute=None):

		xpath = "sbml:species"
		if self.__model.sbmlLevel == 1:
			xpath += "[@name='%s']" % self.getSbmlId()
		else:
			xpath += "[@id='%s']" % self.getSbmlId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])

	def getSpecies(self):

		all_species = []
		for species in self.__model.listOfSpecies:
			if species.getCompartment() == self:
				all_species.append(species)

		return all_species


	def getNbSpecies(self):

		count = 0
		for species in self.__model.listOfSpecies:
			if species.getCompartment() == self:
				count += 1
		return count

	def getByXPath(self, xpath):

		if len(xpath) == 0:
			return self

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@size":
			return self.getValue()

		elif xpath[0] == "@name":
			return self.getName()

		elif xpath[0] == "@id":
			return self.getSbmlId()

	def setByXPath(self, xpath, object):

		if len(xpath) == 0:
			return InvalidXPath("/".join(xpath))

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@size":
			return self.setValue(object)

		elif xpath[0] == "@name":
			return self.setName(object)

		elif xpath[0] == "@id":
			return self.setSbmlId(object)

	def getXPath(self, attribute=None):

		xpath = "sbml:compartment"
		if self.__model.sbmlLevel == 1:
			xpath += "[@name='%s']" % self.getSbmlId()
		else:
			xpath += "[@id='%s']" % self.getSbmlId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""

from libsignetsim import Model, Experiment, ModelVsTimeseriesOptimization

from unittest import TestCase


class TestOptimization(TestCase):
	""" Tests high level functions """

	def testOptimizeMichaelisMenten(self):

		reference_times = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]
		reference_data = [
			0.0, 1.897738450655051, 3.756955074247717, 5.56216742392093, 7.287473256778753, 8.886428870529057,
			10.27109002756082, 11.2877220979055, 11.80436967759331, 11.95991169490768, 11.99256430291778,
			11.99865017124369, 11.99975594720175, 11.99995589772631, 11.99999202708974, 11.99999855444305,
			11.99999973767934, 11.99999993029574, 11.99999999125534, 11.99999999568526, 12.00000000008859
		]

		m = Model()
		m.setName("Enzymatic Reaction")

		e = m.listOfSpecies.new("E")
		s = m.listOfSpecies.new("S")
		p = m.listOfSpecies.new("P")

		m.listOfParameters.new("vmax")
		m.listOfParameters.new("km")

		r = m.listOfReactions.new("Enzymatic reaction")
		r.listOfReactants.add(s)
		r.listOfModifiers.add(e)
		r.listOfProducts.add(p)
		r.kineticLaw.setPrettyPrintMathFormula("vmax*E*S/(km+S)")

		e.setValue(10)
		s.setValue(12)
		p.setValue(0)

		# Building the experiment
		experiment = Experiment()
		condition = experiment.createCondition()
		for i, data in enumerate(reference_data):
			condition.addObservation(reference_times[i], 'P', data)

		selected_parameters = []
		for parameter in m.listOfParameters:
			selected_parameters.append((parameter, 1, 1e-6, 1e+6, 4))

		fit = ModelVsTimeseriesOptimization(
			workingModel=m,
			list_of_experiments=[experiment],
			parameters_to_fit=selected_parameters,
			nb_procs=2
		)

		score = fit.runOptimization(2)
		parameters = fit.readOptimizationOutput()

		self.assertEqual(score, 0.001)
		self.assertAlmostEqual(parameters[m.listOfParameters.getBySbmlId('vmax')], 0.211, delta=1e-3)
		self.assertAlmostEqual(parameters[m.listOfParameters.getBySbmlId('km')], 1.233, delta=1e-3)

	def testOptimizeMichaelisMentenLocalParameters(self):

		# Reference data to fit against
		reference_times = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]
		reference_data = [
			0.0, 1.897738450655051, 3.756955074247717, 5.56216742392093, 7.287473256778753, 8.886428870529057,
			10.27109002756082, 11.2877220979055, 11.80436967759331, 11.95991169490768, 11.99256430291778,
			11.99865017124369, 11.99975594720175, 11.99995589772631, 11.99999202708974, 11.99999855444305,
			11.99999973767934, 11.99999993029574, 11.99999999125534, 11.99999999568526, 12.00000000008859
		]

		# Building the model
		m = Model()
		m.setName("Enzymatic Reaction")

		e = m.listOfSpecies.new("E")
		s = m.listOfSpecies.new("S")
		p = m.listOfSpecies.new("P")

		r = m.listOfReactions.new("Enzymatic reaction")
		r.listOfReactants.add(s)
		r.listOfModifiers.add(e)
		r.listOfProducts.add(p)

		r.listOfLocalParameters.new("vmax")
		r.listOfLocalParameters.new("km")

		r.kineticLaw.setPrettyPrintMathFormula("vmax*E*S/(km+S)")

		e.setValue(10)
		s.setValue(12)
		p.setValue(0)

		# Building the experiment
		experiment = Experiment()
		condition = experiment.createCondition()
		for i, data in enumerate(reference_data):
			condition.addObservation(reference_times[i], 'P', data)

		selected_parameters = []
		for parameter in r.listOfLocalParameters:
			selected_parameters.append((parameter, 1, 1e-6, 1e+6, 4))

		fit = ModelVsTimeseriesOptimization(
			workingModel=m,
			list_of_experiments=[experiment],
			parameters_to_fit=selected_parameters,
			nb_procs=2
		)

		score = fit.runOptimization(2)
		parameters = fit.readOptimizationOutput()

		self.assertEqual(score, 0.001)
		self.assertAlmostEqual(parameters[r.listOfLocalParameters.getBySbmlId('vmax')], 0.211, delta=1e-3)
		self.assertAlmostEqual(parameters[r.listOfLocalParameters.getBySbmlId('km')], 1.233, delta=1e-3)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


import libsbml

from libsignetsim.settings.Settings import Settings
from libsignetsim.model.sbml.CVTerm import CVTerm
from libsignetsim.uris.URI import URI
from libsbml import CVTerm as LibsbmlCVTerm
from libsbml import SBO


class SbmlAnnotation(object):


	def __init__(self, model):

		self.__model = model
		self.__cvTerms = []
		self.__sboTerm = None
		self.__sboURI = None

	def readSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		if sbml_object.getCVTerms() is not None:

			for cvterm in sbml_object.getCVTerms():
				t_cvterm = CVTerm(self.__model)
				t_cvterm.readSbml(cvterm, sbml_level, sbml_version)
				self.__cvTerms.append(t_cvterm)

		if sbml_object.isSetSBOTerm():
			self.__sboTerm = int(sbml_object.getSBOTerm())
			self.__sboURI = URI()
			self.__sboURI.setSBO(self.__sboTerm)


	def writeSbml(self, sbml_object, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if self.__sboTerm is not None:
			sbml_object.setSBOTerm(str(self.__sboTerm))

		for cv_term in self.__cvTerms:
			t_cvterm = LibsbmlCVTerm()
			cv_term.writeSbml(t_cvterm, sbml_level, sbml_version)
			sbml_object.addCVTerm(t_cvterm)

	def getSBOTerm(self):
		return self.__sboTerm

	def getSBOTermDescription(self):
		if self.__sboURI is not None:
			return self.__sboURI.getName()
		elif self.__sboTerm is not None:
			self.__sboURI = URI()
			self.__sboURI.setSBO(self.__sboTerm)
			return self.__sboURI.getName()

	def setSBOTerm(self, sbo_term):
		self.__sboTerm = sbo_term

	def getCVTerms(self):
		return self.__cvTerms


	def getHasTaxon(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isHasTaxon():
				res += cv_term.getURIs()
		return res

	def getHasPart(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isHasPart():
				res += cv_term.getURIs()
		return res

	def getHasProperty(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isHasProperty():
				res += cv_term.getURIs()
		return res

	def getHasVersion(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isHasVersion():
				res += cv_term.getURIs()
		return res
	
	def getIs(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIs():
				res += cv_term.getURIs()
		return res

	def getIsDescribedBy(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIsDescribedBy():
				res += cv_term.getURIs()
		return res

	def addIsDesribedBy(self, uri):
		t_cv_term = CVTerm(self.__model)
		t_cv_term.setIsDescribedBy()
		t_cv_term.addURI(uri)
		self.__cvTerms.append(t_cv_term)

	def clearIsDescribedBy(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIsDescribedBy():
				self.__cvTerms.remove(cv_term)

	def getIsEncodedBy(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIsEncodedBy():
				res += cv_term.getURIs()
		return res

	def getIsHomologTo(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIsHomologTo():
				res += cv_term.getURIs()
		return res

	def getIsPartOf(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIsPartOf():
				res += cv_term.getURIs()
		return res

	def getIsPropertyOf(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIsPropertyOf():
				res += cv_term.getURIs()
		return res

	def getIsVersionOf(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isIsVersionOf():
				res += cv_term.getURIs()
		return res

	def getOccursIn(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isOccursIn():
				res += cv_term.getURIs()
		return res

	def getUnknown(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isUnknown():
				res += cv_term.getURIs()
		return res

	def getModelHasInstance(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isModelHasInstance():
				res += cv_term.getURIs()
		return res

	def getModelIs(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isModelIs():
				res += cv_term.getURIs()
		return res

	def getModelIsDerivedFrom(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isModelIsDerivedFrom():
				res += cv_term.getURIs()
		return res

	def getModelIsDescribedBy(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isModelIsDescribedBy():
				res += cv_term.getURIs()
		return res

	def getModelIsInstanceOf(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isModelIsInstanceOf():
				res += cv_term.getURIs()
		return res

	def getModelUnknown(self):
		res = []
		for cv_term in self.__cvTerms:
			if cv_term.isModelUnknown():
				res += cv_term.getURIs()
		return res





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Test of combine archive : run all sedml files from a case of the libsbml test suite

"""

from libsignetsim import CombineArchive
from unittest import TestCase
from os.path import join, dirname


class TestRunSedmls(TestCase):

	def testRunSedmls(self):

		ca = CombineArchive()
		ca.readArchive(join(join(dirname(__file__), "files"), "00001.omex"))
		sedmls = ca.runAllSedmls()
		for sedml in sedmls:
			self.assertTrue(sedml.listOfOutputs.getReports() is not None)

	def testMasterSedml(self):

		ca = CombineArchive()
		ca.readArchive(join(join(dirname(__file__), "files"), "00001.omex"))
		sedml = ca.runMasterSedml()
		self.assertTrue(sedml.listOfOutputs.getReports() is not None)









#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.MathSymbol import MathSymbol
from libsignetsim.settings.Settings import Settings


class Rule(SbmlObject):

	RULE_ALGEBRAIC              =   0
	RULE_ASSIGNMENT             =   1
	RULE_RATE                   =   2
	RULE_UNKNOWN                =   3
	RULE_SPECIES_CONCENTRATION  =   4
	RULE_COMPARTMENT_VOLUME     =   5
	RULE_PARAMETER              =   6

	ruleTypes = {RULE_ALGEBRAIC: "Algebraic rule",
				 RULE_ASSIGNMENT: "Assignment rule",
				 RULE_RATE: "Rate rule",
				 RULE_SPECIES_CONCENTRATION: "Species concentration rule",
				 RULE_COMPARTMENT_VOLUME: "Compartment volume rule",
				 RULE_PARAMETER: "Parameter rule"}


	def __init__ (self, model, objId, rule_type=None):

		self.__model = model
		self.objId = objId

		SbmlObject.__init__(self, model)

		self.ruleType = rule_type


	def readSbml(self, sbml_rule, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		SbmlObject.readSbml(self, sbml_rule, sbml_level, sbml_version)

	def writeSbml(self, sbml_rule, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		SbmlObject.writeSbml(self, sbml_rule, sbml_level, sbml_version)

	def copy(self, obj, sids_subs={}, symbols_subs={}, conversion_factors={}, time_conversion=None):

		SbmlObject.copy(self, obj)
		self.ruleType = obj.ruleType

	def isAssignment(self):

		return self.ruleType == self.RULE_ASSIGNMENT


	def isRate(self):

		return self.ruleType == self.RULE_RATE

	def isAlgebraic(self):

		return self.ruleType == self.RULE_ALGEBRAIC


	def getType(self):

		return self.ruleType


	def getRuleType(self):

		return self.ruleType

	def getRuleTypeDescription(self):

		return self.ruleTypes[self.ruleType]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function
from __future__ import division

from libsignetsim.model.Model import Model
from libsignetsim.model.Variable import Variable
from libsignetsim.model.sbml.FunctionDefinition import FunctionDefinition
from libsignetsim.model.sbml.UnitDefinition import UnitDefinition
from libsignetsim.model.math.sympy_shortcuts import SympySymbol
from libsignetsim.model.sbml.SbmlObject import SbmlObject

class ModelInstance(Model):
	""" Sbml model class """

	PREFIX_PATTERN = "%s__"
	DEBUG = False

	def __init__(self, model, document):
		""" Constructor of model class """

		Model.__init__(self, parent_doc=document)
		self.__mainModel = model
		self.__document = document
		self.__submodelInstances = {}
		self.sbmlLevel = model.sbmlLevel
		self.sbmlVersion = model.sbmlVersion
		self.objectsDictionnary = {}
		self.variablesDictionnary = {}
		self.deletions = []

		self.submodel_sids_subs = {}
		self.submodel_symbols_subs = {}
		self.submodel_usids_subs = {}
		self.submodel_timeConversionFactor = {}
		self.submodel_extentConversionFactor = {}
		self.conv_factors = {}

		if self.DEBUG:
			print("\n\n> Instanciating model %s, parent doc is %s" % (model.getNameOrSbmlId(), document.documentFilename))


		if len(self.__mainModel.listOfSubmodels) > 0:

			# Instanciating submodels depth first
			for submodel in self.__mainModel.listOfSubmodels:
				t_submodel_instance = submodel.getModelInstance()
				self.__submodelInstances.update({submodel.getSbmlId(): t_submodel_instance})
				self.submodel_sids_subs.update({submodel.getSbmlId(): {}})
				self.submodel_symbols_subs.update({submodel.getSbmlId(): {}})
				self.submodel_usids_subs.update({submodel.getSbmlId(): {}})

				for deletion in submodel.listOfDeletions:
					self.deletions.append(deletion.getDeletionObjectFromInstance(t_submodel_instance))

				for variable in t_submodel_instance.listOfVariables:
					if variable not in self.deletions:
						self.submodel_sids_subs[submodel.getSbmlId()].update({
							variable.getSbmlId(): (self.PREFIX_PATTERN % submodel.getSbmlId()) + variable.getSbmlId()
						})

						if variable.isParameter() and variable.localParameter:
							old_symbol = variable.symbol.getInternalMathFormula()
							new_symbol = SympySymbol("_local_%d_%s" % (variable.reaction.objId,
								(self.PREFIX_PATTERN % submodel.getSbmlId())
								+ variable.getSbmlId()
							))
						else:
							old_symbol = variable.symbol.getInternalMathFormula()
							new_symbol = SympySymbol(
								(self.PREFIX_PATTERN % submodel.getSbmlId())
								+ variable.symbol.getInternalMathFormula().name
							)
						self.submodel_symbols_subs[submodel.getSbmlId()].update({old_symbol: new_symbol})

				for function_definition in t_submodel_instance.listOfFunctionDefinitions:
					if function_definition not in self.deletions:
						self.submodel_sids_subs[submodel.getSbmlId()].update({
							function_definition.getSbmlId(): (self.PREFIX_PATTERN % submodel.getSbmlId()) + function_definition.getSbmlId()
						})

						old_symbol = SympySymbol(function_definition.getSbmlId())
						new_symbol = SympySymbol(
							(self.PREFIX_PATTERN % submodel.getSbmlId())
							+ function_definition.getSbmlId()
						)
						self.submodel_symbols_subs[submodel.getSbmlId()].update({old_symbol: new_symbol})

				for unit_definition in t_submodel_instance.listOfUnitDefinitions:
					if unit_definition not in self.deletions:
						self.submodel_usids_subs[submodel.getSbmlId()].update({
							unit_definition.getSbmlId(): (self.PREFIX_PATTERN % submodel.getSbmlId()) + unit_definition.getSbmlId()
						})


				if submodel.hasExtentConversionFactor():
					self.submodel_extentConversionFactor.update(
						{submodel.getSbmlId(): submodel.getExtentConversionFactor()}
					)

				else:
					self.submodel_extentConversionFactor.update({submodel.getSbmlId(): None})

				if submodel.hasTimeConversionFactor():
					self.submodel_timeConversionFactor.update(
						{submodel.getSbmlId(): submodel.getTimeConversionFactor()}
					)
					self.submodel_symbols_subs[submodel.getSbmlId()].update(
						{SympySymbol("_time_"): SympySymbol("_time_") / submodel.getTimeConversionFactor().getInternalMathFormula()}
					)


				else:
					self.submodel_timeConversionFactor.update({submodel.getSbmlId(): None})


			self.findReplacements()

			if self.DEBUG:
				print(">> Sids dictionnaries")
				print(self.submodel_sids_subs)
				print(">> Symbols dictionnaries")
				print(self.submodel_symbols_subs)

		self.mergeModels()

		if self.DEBUG:
			print(">> Model's variables : ")
			print(self.listOfVariables.sbmlIds())
			print(self.listOfVariables.symbols())

			print("> Returning instance %s\n" % model.getSbmlId())

	def findReplacements(self):

		for sbmlobject in self.__mainModel.listOfSbmlObjects:

			if isinstance(sbmlobject, SbmlObject) and sbmlobject.hasReplacedElements():

				for replaced_element in sbmlobject.getListOfReplacedElements():

					replaced_object = replaced_element.getReplacedElementObjectFromInstance(self)
					self.deletions.append(replaced_object)

					if isinstance(sbmlobject, Variable):

						old_string = (self.PREFIX_PATTERN % replaced_element.getSubmodelRef()) + replaced_object.getSbmlId()
						new_string = sbmlobject.getSbmlId()

						for old, new in list(self.submodel_sids_subs[replaced_element.getSubmodelRef()].items()):
							if new == old_string:
								self.submodel_sids_subs[replaced_element.getSubmodelRef()].update({
									old: new_string
								})

						if replaced_object.isParameter() and replaced_object.localParameter:
							old_symbol = SympySymbol("_local_%d_%s" % (replaced_object.reaction.objId, old_string))
						else:
							old_symbol = SympySymbol(old_string)

						new_symbol = SympySymbol(new_string)

						for old, new in list(self.submodel_symbols_subs[replaced_element.getSubmodelRef()].items()):
							if new == old_symbol:
								self.submodel_symbols_subs[replaced_element.getSubmodelRef()].update({
									old: new_symbol
								})

						if replaced_element.getConversionFactor() is not None:
							self.conv_factors.update({SympySymbol(sbmlobject.getSbmlId()): SympySymbol(
								replaced_element.getConversionFactor())})

					if isinstance(sbmlobject, FunctionDefinition):

						old_string = (self.PREFIX_PATTERN % replaced_element.getSubmodelRef()) + replaced_object.getSbmlId()
						new_string = sbmlobject.getSbmlId()

						for old, new in list(self.submodel_sids_subs[replaced_element.getSubmodelRef()].items()):
							if new == old_string:
								self.submodel_sids_subs[replaced_element.getSubmodelRef()].update({
									old: new_string
								})


						old_symbol = SympySymbol(old_string)
						new_symbol = SympySymbol(new_string)

						for old, new in list(self.submodel_symbols_subs[replaced_element.getSubmodelRef()].items()):
							if new == old_symbol:
								self.submodel_symbols_subs[replaced_element.getSubmodelRef()].update({
									old: new_symbol
								})

					if isinstance(sbmlobject, UnitDefinition):

						old_string = (self.PREFIX_PATTERN % replaced_element.getSubmodelRef()) + replaced_object.getSbmlId()
						new_string = sbmlobject.getSbmlId()

						for old, new in list(self.submodel_usids_subs[replaced_element.getSubmodelRef()].items()):
							if new == old_string:
								self.submodel_usids_subs[replaced_element.getSubmodelRef()].update({
									old: new_string
								})




			if isinstance(sbmlobject, SbmlObject) and sbmlobject.isReplaced():

				replacing_object = sbmlobject.isReplacedBy().getReplacingElementObjectFromInstance(self)
				self.deletions.append(sbmlobject)

				if isinstance(sbmlobject, Variable) or isinstance(sbmlobject, FunctionDefinition):
					old_string = sbmlobject.getSbmlId()
					new_string = replacing_object.getSbmlId()

					if new_string in list(self.submodel_sids_subs[sbmlobject.isReplacedBy().getSubmodelRef()].keys()):
						self.submodel_sids_subs[sbmlobject.isReplacedBy().getSubmodelRef()].update({new_string: old_string})

					old_symbol = SympySymbol(old_string)
					new_symbol = SympySymbol(new_string)

					if new_symbol in list(self.submodel_symbols_subs[sbmlobject.isReplacedBy().getSubmodelRef()].keys()):
						self.submodel_symbols_subs[sbmlobject.isReplacedBy().getSubmodelRef()].update({new_symbol: old_symbol})

				if isinstance(sbmlobject, UnitDefinition):
					old_string = sbmlobject.getSbmlId()
					new_string = replacing_object.getSbmlId()

					if new_string in list(self.submodel_usids_subs[sbmlobject.isReplacedBy().getSubmodelRef()].keys()):
						self.submodel_usids_subs[sbmlobject.isReplacedBy().getSubmodelRef()].update({new_string: old_string})


	def mergeModels(self):

		# This function copy the main model and the submodels into the instanciated model, one element at a time

		self.listOfUnitDefinitions.copy(
			self.__mainModel.listOfUnitDefinitions,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfUnitDefinitions.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfUnitDefinitions,
				deletions=self.deletions,
				usids_subs=self.submodel_usids_subs[submodel.getSbmlId()]
			)

		self.listOfFunctionDefinitions.copy(
			self.__mainModel.listOfFunctionDefinitions,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfFunctionDefinitions.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfFunctionDefinitions,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()]
			)


		self.listOfCompartments.copy(
			self.__mainModel.listOfCompartments,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfCompartments.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfCompartments,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()],
				symbols_subs=self.submodel_symbols_subs[submodel.getSbmlId()],
				usids_subs=self.submodel_usids_subs[submodel.getSbmlId()]
			)

		self.listOfParameters.copy(
			self.__mainModel.listOfParameters,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfParameters.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfParameters,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()],
				symbols_subs=self.submodel_symbols_subs[submodel.getSbmlId()],
				usids_subs=self.submodel_usids_subs[submodel.getSbmlId()]
			)

		self.listOfSpecies.copy(
			self.__mainModel.listOfSpecies,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfSpecies.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfSpecies,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()],
				symbols_subs=self.submodel_symbols_subs[submodel.getSbmlId()],
				usids_subs=self.submodel_usids_subs[submodel.getSbmlId()]
			)

		self.listOfReactions.copy(
			self.__mainModel.listOfReactions,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfReactions.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfReactions,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()],
				symbols_subs=self.submodel_symbols_subs[submodel.getSbmlId()],
				usids_subs=self.submodel_usids_subs[submodel.getSbmlId()],
				conversion_factors=self.conv_factors,
				time_conversion=self.submodel_timeConversionFactor[submodel.getSbmlId()],
				extent_conversion=self.submodel_extentConversionFactor[submodel.getSbmlId()]
			)

		self.listOfInitialAssignments.copy(
			self.__mainModel.listOfInitialAssignments,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfInitialAssignments.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfInitialAssignments,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()],
				symbols_subs=self.submodel_symbols_subs[submodel.getSbmlId()],
				conversion_factors=self.conv_factors
			)

		self.listOfRules.copy(
			self.__mainModel.listOfRules,
			deletions=self.deletions,
		)
		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfRules.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfRules,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()],
				symbols_subs=self.submodel_symbols_subs[submodel.getSbmlId()],
				conversion_factors=self.conv_factors,
				time_conversion=self.submodel_timeConversionFactor[submodel.getSbmlId()],
			)

		self.listOfEvents.copy(
			self.__mainModel.listOfEvents,
			deletions=self.deletions,
		)

		for submodel in self.__mainModel.listOfSubmodels:
			self.listOfEvents.copy(
				self.__submodelInstances[submodel.getSbmlId()].listOfEvents,
				deletions=self.deletions,
				sids_subs=self.submodel_sids_subs[submodel.getSbmlId()],
				symbols_subs=self.submodel_symbols_subs[submodel.getSbmlId()],
				conversion_factors=self.conv_factors,
				time_conversion=self.submodel_timeConversionFactor[submodel.getSbmlId()],
			)

		for variable in self.__mainModel.listOfVariables:
			if self.listOfVariables.containsSbmlId(variable.getSbmlId()):
				self.variablesDictionnary.update({
					variable: self.listOfVariables.getBySbmlId(variable.getSbmlId())
				})

		for submodel in self.__mainModel.listOfSubmodels:
			submodel_instance = self.__submodelInstances[submodel.getSbmlId()]

			for variable_def, variable_instance in list(submodel_instance.variablesDictionnary.items()):
				sids_subs = self.submodel_sids_subs[submodel.getSbmlId()]

				if (
					variable_instance.getSbmlId() in list(sids_subs.keys())
					and self.listOfVariables.containsSbmlId(sids_subs[variable_instance.getSbmlId()])
				):
					self.variablesDictionnary.update({
						variable_def: self.listOfVariables.getBySbmlId(
							sids_subs[variable_instance.getSbmlId()]
						)
					})

	def getSubmodelInstance(self, submodel_ref):
		return self.__submodelInstances[submodel_ref]

	def getInstanceVariable(self, variable):
		return self.variablesDictionnary[variable]

	def getInstanceVariableByXPath(self, xpath):
		variable = self.__mainModel.parentDoc.getByXPath(xpath)
		return self.getInstanceVariable(variable)

	def getDefinitionVariable(self, variable):
		res = []
		for key, value in list(self.variablesDictionnary.items()):
			if value == variable:
				res.append(key)
		return res

	def getDefinitionVariableByXPath(self, xpath):
		variable = self.parentDoc.getByXPath(xpath, instance=True)
		return self.getDefinitionVariable(variable)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for the test cases from the SED-ML files
	in the SBmL Test Suite

"""
from __future__ import print_function

from builtins import str
from builtins import object
from libsignetsim.tests.sbmltestsuite.TestSuiteCase import TestSuiteCase
from libsignetsim import Settings

from os.path import join, exists
from os import mkdir


class TestSuite(object):
	""" Tests SED-ML semantic test cases """

	TODO_CASES = [1490]
	TODO_VERSIONS = []
	TODO_TAGS = []

	INCOMPATIBLE_CASES = []

	INCOMPATIBLE_TAGS = [
		'CSymbolDelay'
	]

	# FastReations cases
	INCOMPATIBLE_CASES += [
		986, 987, 988,
		# <<<<<<< Updated upstream
		# =======
		# 		1282,# Case with boolean conversion
		# 		# 1490, 1491, 1494, # Cases with function returning a boolean (but not of a boolean type, since they are functions
		# >>>>>>> Stashed changes
		1398,# Variable stoichiometry, assigned by an event. Not today
		1399,# Two possible choices for initial values, not sure how to choose
		1565,# Problem evaluating sec(0.5) ??!! Also, takes ages
		1568,# Piecewise logic in a conservation law
		1569,# Four possible choices for initial values, not sure again
		1570,# Piecewise login, again
		1571,# Different output type for solve, should be easy
		1572, #Two possible initial conditions
	]

	# RandomEventExecution : This case count twice as much events as it should...
	INCOMPATIBLE_CASES += [1590]

	# VolumeConcentrationRates cases, something to do with event delay...
	INCOMPATIBLE_CASES += [1507, 1508, 1511]

	VERSION_INCOMPATIBLE_TAGS = {}

	COMPATIBLE_PACKAGES = ['comp']

	FAIL_ON_EXCEPTION = True

	def __init__(self, version):

		self.TODO_VERSIONS.append(version)
		self.testCasesPath = None
		self.testCasesTags = {}
		self.testCasesVersions = {}

		self.testExport = True
		Settings.verbose = 0

	def testSuiteRun(self):

		self.testCasesPath = Settings.sbmlTestCasesPath
		if not exists(join(Settings.tempDirectory, "test-suite-results")):
			mkdir(join(Settings.tempDirectory, "test-suite-results"))


		self.loadTestCasesInfo()
		return self.runTestCases()

	def loadTestCasesInfo(self, path=None):
		""" Loads cases info from the .cases-tags-map """


		cases_tags_map_file = join(self.testCasesPath,
											".cases-tags-map")

		if exists(cases_tags_map_file):
			cases_tags_map = open(cases_tags_map_file)
			tags_list = []
			for i, line in enumerate(cases_tags_map.readlines()):

				if i == 0:
					# List of tags. Ignoring
					tags_list = line.strip().split()

				elif line.startswith('+'):
					# Some kind of comment ?
					pass

				elif line.strip() == "README.md":
					pass

				else:
					case_data = line.strip().split()
					case_id = int(case_data[0])
					tags = []
					versions = []
					for data in case_data[1:]:
						if data in tags_list:
							tags.append(data)
						else:
							versions.append(data)

					self.testCasesTags.update({case_id: tags})
					self.testCasesVersions.update({case_id: versions})
			cases_tags_map.close()

	def runTestCases(self):

		nb_success = 0
		nb_cases = 0
		for case_id, case_tags in list(self.testCasesTags.items()):

			compatible = True
			if self.TODO_TAGS != []:
				todo_tag = False
			else:
				todo_tag = True
			for tag in case_tags:

				if tag.strip() in self.TODO_TAGS:
					todo_tag = True

				if tag.strip() in self.INCOMPATIBLE_TAGS:
					compatible = False

				elif (':' in tag.strip()
					  and tag.strip().split(':')[0] not in self.COMPATIBLE_PACKAGES):
					compatible = False

				for todo_version in self.TODO_VERSIONS:
					if todo_version in list(self.VERSION_INCOMPATIBLE_TAGS.keys()) and tag.strip() in self.VERSION_INCOMPATIBLE_TAGS[todo_version]:
						compatible = False


			if (compatible
				and case_id not in self.INCOMPATIBLE_CASES
				and todo_tag
				and (self.TODO_CASES == [] or case_id in self.TODO_CASES)
			):
				(t_success, t_cases) = self.runCase(case_id)
				nb_success += t_success
				nb_cases += t_cases


		if nb_cases > 0:
			print("\n> %d success out of %d tests (%.0f%%)" % (nb_success, nb_cases, nb_success*100/nb_cases))
		return nb_cases == nb_success

	def runCase(self, case):

		nb_cases = 0
		nb_success = 0

		for versions in self.testCasesVersions[case]:
			if self.TODO_VERSIONS == [] or versions in self.TODO_VERSIONS:

				print("> Running case %05d (%s)" % (case, str(self.TODO_VERSIONS)))

				level_version = versions.split('.')
				level = int(level_version[0])
				version = int(level_version[1])

				nb_cases += 1
				if self.FAIL_ON_EXCEPTION:
					if Settings.verbose >= 1 or Settings.verboseTiming >= 1:
						print("")

					test = TestSuiteCase(case, str(level), str(version), test_export=self.testExport)
					if test.run():
						nb_success += 1

				else:
					try:
						if Settings.verbose >= 1 or Settings.verboseTiming >= 1:
							print("")

						test = TestSuiteCase(case, str(level), str(version), test_export=self.testExport)
						if test.run():
							nb_success += 1
					except Exception as e:
						print(">> case %d, %dv%d : ERROR (%s)" % (int(case), level, version, e))

		return nb_success, nb_cases



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.sedml.HasId import HasId
from libsignetsim.settings.Settings import Settings


class Curve(SedBase, HasId):

	def __init__(self, document):

		SedBase.__init__(self, document)
		HasId.__init__(self, document)

		self.__document = document
		self.__xDataReference = None
		self.__yDataReference = None
		self.__logX = False
		self.__logY = False

	def readSedml(self, curve, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, curve, level, version)
		HasId.readSedml(self, curve, level, version)

		if curve.isSetXDataReference():
			self.__xDataReference = curve.getXDataReference()

		if curve.isSetYDataReference():
			self.__yDataReference = curve.getYDataReference()

		if curve.isSetLogX():
			self.__logX = curve.getLogX()

		if curve.isSetLogY():
			self.__logY = curve.getLogY()

	def writeSedml(self, curve, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, curve, level, version)
		HasId.writeSedml(self, curve, level, version)

		if self.__xDataReference is not None:
			curve.setXDataReference(self.__xDataReference)

		if self.__yDataReference is not None:
			curve.setYDataReference(self.__yDataReference)

		if self.__logX is not None:
			curve.setLogX(self.__logX)

		if self.__logY is not None:
			curve.setLogY(self.__logY)

	def getXDataReference(self):
		return self.__xDataReference

	def getYDataReference(self):
		return self.__yDataReference

	def getXData(self):
		data_x = self.__document.listOfDataGenerators.getDataGenerator(self.__xDataReference)
		return data_x.getData()

	def getYData(self):
		data_y = self.__document.listOfDataGenerators.getDataGenerator(self.__yDataReference)
		return data_y.getData()

	def getData(self):
		xs = self.getXData()
		ys = self.getYData()

		data = []
		for i, x in enumerate(xs):
			data.append((x, ys[i]))
		return data

	def getLogX(self):
		return self.__logX

	def getLogY(self):
		return self.__logY

	def setXDataReference(self, x_data_reference):
		self.__xDataReference = x_data_reference

	def setXData(self, x_data):
		self.__xDataReference = x_data.getId()

	def setYDataReference(self, y_data_reference):
		self.__yDataReference = y_data_reference

	def setYData(self, y_data):
		self.__yDataReference = y_data.getId()

	def setLogX(self, log_x):
		self.__logX = log_x

	def setLogY(self, log_y):
		self.__logY = log_y

	def getDataToGenerate(self):

		return [
			self.__document.listOfDataGenerators.getDataGenerator(self.__xDataReference),
			self.__document.listOfDataGenerators.getDataGenerator(self.__yDataReference)
		]

	def build(self, fig, plot, curve_id, print_ynames):

		data_x = self.__document.listOfDataGenerators.getDataGenerator(self.__xDataReference)
		data_y = self.__document.listOfDataGenerators.getDataGenerator(self.__yDataReference)

		# print "X : %s" % str(data_x.getData())
		# print "Y : %s" % str(data_y.getData())
		if not print_ynames:
			fig.plot(plot, curve_id, data_x.getData(), data_y.getData())
		else:
			fig.plot(plot, curve_id, data_x.getData(), data_y.getData(), y_name=self.getYAxisTitle())


	def getXAxisTitle(self):
		return self.__document.listOfDataGenerators.getDataGenerator(self.__xDataReference).getName()

	def getYAxisTitle(self):
		return self.__document.listOfDataGenerators.getDataGenerator(self.__yDataReference).getName()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.DimensionDescription import DimensionDescription
from libsignetsim.numl.TupleDescription import TupleDescription
from libsignetsim.numl.AtomicDescription import AtomicDescription
from libsignetsim.settings.Settings import Settings

class CompositeDescription (DimensionDescription):

	def __init__(self, document, name=None, index_type="string"):
		DimensionDescription.__init__(self, document, name)
		self.__document = document
		self.__content = None
		self.__indexType = index_type
		self.__ontologyTerm = None

	def createCompositeDescription(self, name=None, index_type="string"):

		from libsignetsim.numl.CompositeDescription import CompositeDescription
		self.__content = CompositeDescription(self.__document, name, index_type)
		self.__content.setId("%s_0" % self.getId())
		return self.__content

	def createTupleDescription(self, name=None):
		self.__content = TupleDescription(self.__document, name)
		self.__content.setId("%s_0" % self.getId())
		return self.__content

	def createAtomicDescription(self, name=None, value_type="double"):
		self.__content = AtomicDescription(self.__document, name, value_type)
		self.__content.setId("%s_0" % self.getId())
		return self.__content

	def readNuML(self, composite_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		DimensionDescription.readNuML(self, composite_description)

		if composite_description.isContentCompositeDescription():
			from libsignetsim.numl.CompositeDescription import CompositeDescription
			self.__content = CompositeDescription(self.__document)
			self.__content.readNuML(composite_description[0], level, version)

		elif composite_description.isContentTupleDescription():
			self.__content = TupleDescription(self.__document)
			self.__content.readNuML(composite_description[0], level, version)

		elif composite_description.isContentAtomicDescription():
			self.__content = AtomicDescription(self.__document)
			self.__content.readNuML(composite_description[0], level, version)

		self.__indexType = composite_description.getIndexType()
		self.__ontologyTerm = self.__document.listOfOntologyTerms.getByOntologyTerm(composite_description.getOntologyTerm())

	def writeNuML(self, composite_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		DimensionDescription.writeNuML(self, composite_description)

		if self.__indexType is not None:
			composite_description.setIndexType(self.__indexType)

		if self.__ontologyTerm is not None:
			composite_description.setOntologyTerm(self.__ontologyTerm.getId())

		if self.__content is not None:
			t_content = None
			from libsignetsim.numl.CompositeDescription import CompositeDescription
			if isinstance(self.__content, CompositeDescription):
				t_content = composite_description.createCompositeDescription()

			elif isinstance(self.__content, TupleDescription):
				t_content = composite_description.createTupleDescription()

			elif isinstance(self.__content, AtomicDescription):
				t_content = composite_description.createAtomicDescription()

			if t_content is not None:
				self.__content.writeNuML(t_content, level, version)

	def getContent(self):
		return self.__content


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function
from __future__ import division

from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyInteger, SympyMul, SympyPow
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs
from sympy import pretty, Equality, Symbol


class InitialAssignment(SbmlObject, HasParentObj):
	""" Initial assignment definition """

	def __init__(self, model, parent_obj, obj_id, math_only=False):

		self.__model = model
		self.objId = obj_id

		HasParentObj.__init__(self, parent_obj)

		# For math submodels, where objects are not sbml objects
		self.mathOnly = math_only
		if not self.mathOnly:
			SbmlObject.__init__(self, model)

		self.__definition = MathFormula(self.__model, MathFormula.MATH_ASSIGNMENTRULE)
		self.__var = None

	def readSbml(self, initial_assignment, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads an initial assignment from a sbml file """

		SbmlObject.readSbml(self, initial_assignment, sbml_level, sbml_version)

		if self.__model.listOfVariables.containsSbmlId(initial_assignment.getSymbol()):
			self.__var = self.__model.listOfVariables.getBySbmlId(initial_assignment.getSymbol())
			self.getVariable().setInitialAssignmentBy(self)

		self.__definition.readSbml(initial_assignment.getMath(), sbml_level, sbml_version)

		if self.__definition.getInternalMathFormula() is not None:

			if self.getVariable().isConcentration():
				self.__definition.setInternalMathFormula(
					SympyMul(self.__definition.getInternalMathFormula(),
							self.getVariable().getCompartment().symbol.getInternalMathFormula()))

	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes an initial assignment to a sbml file """

		sbml_initial_assignment = sbml_model.createInitialAssignment()

		SbmlObject.writeSbml(self, sbml_initial_assignment, sbml_level, sbml_version)

		sbml_initial_assignment.setSymbol(self.__var.getSbmlId())

		if self.__definition.getInternalMathFormula() is not None:

			t_definition = MathFormula(self.__model, MathFormula.MATH_ASSIGNMENTRULE)
			t_definition.setInternalMathFormula(self.__definition.getInternalMathFormula())

			if self.getVariable().isConcentration():
				t_definition.setInternalMathFormula(
					SympyMul(self.__definition.getInternalMathFormula(),
							SympyPow(self.getVariable().getCompartment().symbol.getInternalMathFormula(),
										SympyInteger(-1))))

			sbml_initial_assignment.setMath(t_definition.getSbmlMathFormula(sbml_level, sbml_version))

	def copy(self, obj, sids_subs={}, symbols_subs={}, conversion_factors={}):
		SbmlObject.copy(self, obj)

		if obj.getVariable().getSbmlId() in list(sids_subs.keys()):
			self.__var = self.__model.listOfVariables.getBySbmlId(sids_subs[obj.getVariable().getSbmlId()])
		else:
			self.__var = self.__model.listOfVariables.getBySbmlId(obj.getVariable().getSbmlId())

		self.getVariable().setInitialAssignmentBy(self)

		t_convs = {}
		for var, conversion in list(conversion_factors.items()):
			t_convs.update({var: var/conversion})

		if obj.getDefinition() is not None:
			t_definition = unevaluatedSubs(obj.getDefinition().getInternalMathFormula(), symbols_subs)
			t_definition = unevaluatedSubs(t_definition, t_convs)

			t_var_symbol = unevaluatedSubs(obj.getVariable().symbol.getInternalMathFormula(), symbols_subs)

			if t_var_symbol in conversion_factors:
				t_definition *= conversion_factors[t_var_symbol]

			self.__definition.setInternalMathFormula(t_definition)

	def copySubmodel(self, obj):
		if obj.getDefinition() is not None:
			self.__definition.setInternalMathFormula(obj.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula())
		self.__var = self.__model.listOfVariables.getBySymbol(obj.getVariable().symbol.getSymbol())

	def getVariable(self):
		return self.__var

	def setVariable(self, variable):

		if self.__var is not None:
			self.getVariable().unsetInitialAssignmentBy()

		self.__var = variable
		self.getVariable().setInitialAssignmentBy(self)

	def getRawDefinition(self, rawFormula=False):

		# if self.__definition.getInternalMathFormula() is not None:
		formula = self.__definition.getInternalMathFormula()
		if formula is not None:
			if not rawFormula and self.getVariable().isConcentration():
				formula /= self.getVariable().getCompartment().symbol.getInternalMathFormula()

			if not rawFormula:
				subs = {}
				for species in self.__model.listOfSpecies:
					if species.isConcentration():
						subs.update({species.symbol.getInternalMathFormula(rawFormula=True): species.symbol.getInternalMathFormula()})
				formula = unevaluatedSubs(formula, subs)

		return formula

	def getDefinition(self, rawFormula=False):

		# if self.__definition is not None:
		math_formula = MathFormula(self.__model, MathFormula.MATH_ASSIGNMENTRULE)
		math_formula.setInternalMathFormula(self.getRawDefinition(rawFormula=rawFormula))

		return math_formula

	def getRuleTypeDescription(self):
		return "Initial assignment"

	def setPrettyPrintDefinition(self, definition, rawFormula=False):

		if self.getVariable().isConcentration() and not rawFormula:
			t_comp = self.getVariable().getCompartment()
			t_math_formula = MathFormula(self.__model, MathFormula.MATH_ASSIGNMENTRULE)
			t_math_formula.setPrettyPrintMathFormula(definition, rawFormula=rawFormula)
			self.__definition.setInternalMathFormula(
				t_math_formula.getInternalMathFormula() * t_comp.symbol.getInternalMathFormula())

		else:
			self.__definition.setPrettyPrintMathFormula(definition, rawFormula=rawFormula)

	def getPrettyPrintDefinition(self):

		if self.__definition.getInternalMathFormula() is not None:
			if self.getVariable().isSpecies() and not self.getVariable().hasOnlySubstanceUnits:
				t_comp = self.getVariable().getCompartment()
				t_math_formula = MathFormula(self.__model, MathFormula.MATH_ASSIGNMENTRULE)
				t_math_formula.setInternalMathFormula(
					self.__definition.getInternalMathFormula()/t_comp.symbol.getInternalMathFormula()
				)
				return t_math_formula.getPrettyPrintMathFormula()

			else:
				return self.__definition.getPrettyPrintMathFormula()


	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		if self.__definition.getInternalMathFormula() is not None:
			self.__definition.renameSbmlId(old_sbml_id, new_sbml_id)


	def containsVariable(self, variable):

		if self.__definition.getInternalMathFormula() is not None:
			return (variable.symbol.getInternalMathFormula() in self.__definition.getInternalMathFormula().atoms()
					or (variable.isSpecies() and SympySymbol("_speciesForcedConcentration_%s_" % str(variable.symbol.getInternalMathFormula())) in self.__definition.getInternalMathFormula().atoms())
					or variable.symbol.getInternalMathFormula() == self.getVariable().symbol.getInternalMathFormula())
		else:
			return False

	def getByXPath(self, xpath):

		if len(xpath) == 0:
			return self

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@value":
			return self.getValue()

		elif xpath[0] == "@name":
			return self.getName()

		elif xpath[0] == "@id":
			return self.getSbmlId()


	def getXPath(self, attribute=None):

		xpath = "sbml:initialAssignment"
		xpath += "[@metaid='%s']" % self.getMetaId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])

	def pprint(self):
		print(
			pretty(
				Equality(
					Symbol(str(self.getVariable().symbol.getDeveloppedInternalMathFormula()) + "_0"),
					self.getDefinition().getDeveloppedInternalMathFormula()
				)
			)

		)

	def isValid(self):

		return (
			self.getVariable() is not None
			and self.__definition.getInternalMathFormula() is not None
			and self.__definition.getDeveloppedInternalMathFormula() is not None
		)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.numl.Dimension import Dimension
from libsignetsim.numl.AtomicValue import AtomicValue
from libsignetsim.settings.Settings import Settings

class TupleValue (Dimension):

	def __init__(self, document, result_component, description):
		Dimension.__init__(self, document, result_component, description)
		self.__document = document
		self.__atomicValues = []

	def createAtomicValue(self, description=None, value=None):

		atomic_value = AtomicValue(self.__document, self.getResultComponent(), description, value)
		self.__atomicValues.append(atomic_value)
		return atomic_value

	def readNuML(self, tuple_value, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		Dimension.readNuML(self, tuple_value, level, version)

		for i in range(tuple_value.size()):
			t_atomic_value = AtomicValue(self.__document, self.getResultComponent(), self.getDescription().getAtomicDescriptions()[i])
			t_atomic_value.readNuML(tuple_value.getAtomicValue(i), level, version)
			self.__atomicValues.append(t_atomic_value)

	def writeNuML(self, tuple_value, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		Dimension.writeNuML(self, tuple_value, level, version)

		for atomic_value in self.__atomicValues:
			t_atomic_value = tuple_value.createAtomicValue()
			atomic_value.writeNuML(t_atomic_value, level, version)

	def getAtomicValues(self):
		return self.__atomicValues


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import XMLNode
from six.moves import reload_module
reload_module(libsbml)


class NewXML(object):

	def __init__(self, document):
		self.__document = document
		self.__xmlNode = None

	def readSedml(self, xml_node, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		self.__xmlNode = xml_node

	def writeSedml(self, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		return self.__xmlNode

	def setFromString(self, xml_string):
		self.__xmlNode = XMLNode.convertStringToXMLNode(xml_string)

	def getAsString(self):
		return XMLNode.convertXMLNodeToString(self.__xmlNode)

	def getXMLNode(self):
		return self.__xmlNode


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Retryiving NCBI Taxonomy URI informations

"""


import requests
from json import loads

class NCBITaxonomyResolver(object):

	URL = "http://rest.ensembl.org/taxonomy/id/%s?"

	def __init__(self, sbo_id):
		self.__id = sbo_id
		self.__json = None
		self.__name = None
		self.__definition = None

	def lookup(self):

		if self.__id is not None:
			self.getJSON()
			self.parseJSON()

	def getJSON(self):

		r = requests.get(self.URL % self.__id, headers={"Content-Type": "application/json"})

		if r.ok:
			self.__json = loads(r.content.decode('utf-8'))

	def parseJSON(self):

		if self.__json is not None:
			self.__name = str(self.__json[u'name'])
			self.__definition = self.__json[u'scientific_name']

	def getName(self):
		return self.__name

	def getDefinition(self):
		return self.__definition


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.Rule import Rule
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.MathFormula import MathFormula

from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyEqual
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs

class AlgebraicRule(Rule):
	""" Class for rate rules """

	def __init__ (self, model, objId):

		self.__model = model
		Rule.__init__(self, model, objId, Rule.RULE_ALGEBRAIC)
		self.__definition = MathFormula(model,
										MathFormula.MATH_ALGEBRAICRULE)


	def readSbml(self, sbml_rule,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		Rule.readSbml(self, sbml_rule, sbml_level, sbml_version)
		self.__definition.readSbml(sbml_rule.getMath(), sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		sbml_rule = sbml_model.createAlgebraicRule()
		Rule.writeSbml(self, sbml_rule, sbml_level, sbml_version)
		sbml_rule.setMath(self.__definition.getSbmlMathFormula(sbml_level,
																sbml_version))


	def copy(self, obj, symbols_subs={}):

		Rule.copy(self, obj)

		t_definition = unevaluatedSubs(obj.getDefinition().getInternalMathFormula(), symbols_subs)
		self.__definition.setInternalMathFormula(t_definition)

	def getRawDefinition(self, rawFormula=False):

		formula = self.__definition.getInternalMathFormula()
		if formula is not None and not rawFormula:
			subs = {}
			for species in self.__model.listOfSpecies:
				if species.isConcentration():
					subs.update({species.symbol.getInternalMathFormula(rawFormula=True): species.symbol.getInternalMathFormula()})
			formula = unevaluatedSubs(formula, subs)

		return formula


	def getDefinition(self, rawFormula=False):

		math_formula = MathFormula(self.__model)
		math_formula.setInternalMathFormula(self.getRawDefinition(rawFormula=rawFormula))

		return math_formula


	# def setDefinition(self, definition):
	# 	self.__definition = definition

	def setPrettyPrintDefinition(self, definition, rawFormula=False):
		self.__definition.setPrettyPrintMathFormula(definition, rawFormula=rawFormula)

	def getPrettyPrintDefinition(self):
		if self.__definition.getInternalMathFormula() is not None:
			return self.__definition.getPrettyPrintMathFormula()


	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		self.__definition.renameSbmlId(old_sbml_id, new_sbml_id)

	def containsVariable(self, variable):
		if self.__definition .getInternalMathFormula() is not None:
			return (variable.symbol.getInternalMathFormula() in self.__definition.getInternalMathFormula().atoms()
					or (variable.isSpecies() and SympySymbol("_speciesForcedConcentration_%s_" % str(variable.symbol.getInternalMathFormula())) in self.__definition.getInternalMathFormula().atoms()))
		else:
			return False

	def isValid(self):
		return (
			self.__definition.getInternalMathFormula() is not None
			and self.__definition.getDeveloppedInternalMathFormula() is not None
		)


#!/usr/bin/env python
""" TestLoadLargeModel.py


	Testing the research for conservation laws, and the subsequent reduction


	Copyright (C) 2016 Vincent Noel (vincent.noel@butantan.gov.br)

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.

"""

from libsignetsim.model.SbmlDocument import SbmlDocument
from libsignetsim.simulation.TimeseriesSimulation import TimeseriesSimulation
from libsignetsim.settings.Settings import Settings

from unittest import TestCase
from os.path import join, dirname, isdir
from os import mkdir, getcwd

class TestLoadLargeModel(TestCase):
	""" Tests high level functions """


	def testLoadGerared(self):

		Settings.verboseTiming = 3

		testfiles_path = join(join(getcwd(), dirname(__file__)), "files")
		sbml_doc = SbmlDocument()
		sbml_doc.readSbmlFromFile(join(testfiles_path, "gerard.xml"))

		# sbml_model = sbml_doc.getModelInstance()
		# sbml_model.build()

		model = sbml_doc.getModelInstance()

		sim = TimeseriesSimulation([model], time_min=0, time_ech=1, time_max=100)
		sim.run()
		t, y = sim.getRawData()[0]

		Settings.verboseTiming = 0


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.model.sbml.UnitDefinition import UnitDefinition
from libsignetsim.settings.Settings import Settings

class HasUnits(object):


	def __init__ (self, model):

		self.__model = model
		self.__unit = None
		self.__builtinUnit = False


	def readSbml(self, sbml_object, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads a parameter from a sbml file """
		if sbml_object.isSetUnits():
			if self.__model.listOfUnitDefinitions.containsSbmlId(sbml_object.getUnits()):
				self.__unit = sbml_object.getUnits()
			elif sbml_level < 3:
				self.__builtinUnit = True
				self.__unit = sbml_object.getUnits()


	def writeSbml(self, sbml_object, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes a parameter to  a sbml file """

		if self.__unit is not None:
			if self.__builtinUnit is False:
				sbml_object.setUnits(self.__unit)
			if self.__builtinUnit is True and sbml_level < 3:
				sbml_object.setUnits(self.__unit)


	def new(self, unit=None):

		if unit is None:
			self.setUnits(unit)
		elif self.__model.getSubstanceUnits() == self.__model.getDefaultSubstanceUnits():
			self.setDefaultSubstanceUnits()
			

	def setUnits(self, unit, prefix=""):
		if unit is not None:
			self.__unit = prefix + unit.getSbmlId()


	def getUnits(self):
		if self.__unit is not None:
			if self.__builtinUnit:
				return self.__unit
			else:
				return self.__model.listOfUnitDefinitions.getBySbmlId(self.__unit)
		else:
			return None


	def setUnitId(self, unit_id, prefix=""):
		if unit_id is not None:
			self.__unit = prefix + unit_id

	def getUnitId(self):
		return self.__unit

	def hasUnits(self):
		return self.__unit is not None


	def setDefaultVolumeUnit(self):
		self.__unit = "volume"


	def copy(self, obj, usids_subs={}):
		if obj.getUnitId() in list(usids_subs.keys()):
			self.__unit = usids_subs[obj.getUnitId()]
		else:
			self.__unit = obj.getUnitId()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""



from libsignetsim.model.sbml.SbmlAnnotation import SbmlAnnotation
from libsignetsim.settings.Settings import Settings
from libsbml import SyntaxChecker
from libsbml import XMLNode
from libsignetsim.model.ModelException import CannotCreateException

class SimpleSbmlObject(object):

	def __init__(self, model):

		self.__model = model
		self.__metaId = None
		self.__notes = None
		self.__annotation = SbmlAnnotation(self)
		self.newMetaId()
		self.__model.listOfSbmlObjects.addSbmlObject(self)

	def new(self, notes=None):

		self.__notes = notes

	def copy(self, obj):

		self.__notes = obj.getNotes()
		self.__model.objectsDictionnary.update({obj.getMetaId(): self.getMetaId()})

	def readSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		if sbml_level >= 2:
			if sbml_object.isSetMetaId():
				self.setMetaId(sbml_object.getMetaId(), force=True)

		if sbml_object.isSetNotes():
			t_notes = sbml_object.getNotes().getChild(0)

			self.__notes = ""
			for note in range(t_notes.getNumChildren()):
				self.__notes += t_notes.getChild(note).toXMLString()

		if sbml_object.isSetAnnotation():
			self.__annotation.readSbml(sbml_object, sbml_level, sbml_version)

	def writeSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):


		if sbml_level >= 2 and self.__metaId is not None:
			sbml_object.setMetaId(str(self.__metaId))

		if self.__notes is not None and self.__notes != "":
			sbml_object.setNotes(self.buildNotes(self.__notes))

		self.__annotation.writeSbml(sbml_object, sbml_level, sbml_version)

	def newMetaId(self):
		self.__metaId = self.__model.listOfSbmlObjects.nextMetaId()

	def setMetaId(self, meta_id, force=False):

		if meta_id is not None:
			t_meta_id = meta_id

			if SyntaxChecker.isValidXMLID(t_meta_id):

				if self.__model.listOfSbmlObjects.containsMetaId(t_meta_id) and force:
				# Here is a case where we probably choosed a meta id for an object, not knowing that the SBML file
				# already had that meta id reserved for another object. So we need to rename the first one, so that
				# we can keep the meta id of the reserved one.
					t_object = self.__model.listOfSbmlObjects.getByMetaId(t_meta_id)

					t_new_meta_id = self.__model.listOfSbmlObjects.nextMetaId()
					while self.__model.listOfSbmlObjects.containsMetaId(t_new_meta_id):
						t_new_meta_id = self.__model.listOfSbmlObjects.nextMetaId()

					t_object.setMetaId(t_new_meta_id)

					self.__metaId = t_meta_id

				else:
					while self.__model.listOfSbmlObjects.containsMetaId(t_meta_id):
						t_meta_id = self.__model.listOfSbmlObjects.nextMetaId()

					self.__metaId = t_meta_id

			else:
				raise CannotCreateException("MetaId is not valid !!")

	def rawSetMetaId(self, meta_id):
		self.__metaId = meta_id
		
	def unsetMetaId(self):
		self.__metaId = None


	def getMetaId(self):
		return self.__metaId

	def getNotes(self):
		return self.__notes

	def setNotes(self, notes):

		if (notes is not None
			and SyntaxChecker.hasExpectedXHTMLSyntax(self.buildNotes(notes))):
			self.__notes = notes


	def buildNotes(self, string):

		t_string = ("<notes><body xmlns=\"http://www.w3.org/1999/xhtml\">"
						+ string
					+ "</body></notes>")

		return XMLNode.convertStringToXMLNode(t_string)

	def getAnnotation(self):
		return self.__annotation


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOfParameters import ListOfParameters
from libsignetsim.model.sbml.container.ListOfSpeciesReference import ListOfSpeciesReference

from libsignetsim.model.sbml.KineticLaw import KineticLaw
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs
from libsignetsim.model.sbml.HasUnits import HasUnits
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.Variable import Variable
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.settings.Settings import Settings
from sympy import zeros
from libsignetsim.model.math.sympy_shortcuts import SympySymbol
from libsignetsim.model.ModelException import InvalidXPath


class Reaction(Variable, SbmlObject, HasUnits, HasParentObj):
	""" Parent class for Sbml reaction """


	def __init__ (self, model, parent_obj, obj_id, name=None, reaction_type=KineticLaw.UNDEFINED):

		self.__model = model
		self.objId = obj_id

		HasUnits.__init__(self, model)
		Variable.__init__(self, model, Variable.REACTION)
		SbmlObject.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)

		self.reversible = True
		self.fast = False
		self.reactionType = reaction_type

		self.kineticLaw = None

		self.listOfReactants = ListOfSpeciesReference(model)
		self.listOfModifiers = ListOfSpeciesReference(model)
		self.listOfProducts = ListOfSpeciesReference(model)
		self.listOfLocalParameters = ListOfParameters(model, self, are_local_parameters=True, reaction=self)


	def new(self, name=None):

		Variable.new(self, name, Variable.REACTION)
		self.kineticLaw = KineticLaw(self.__model, self)
		SbmlObject.new(self)

	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs=[], usids_subs={}, conversion_factors={},
			extent_conversion=None, time_conversion=None):

		HasUnits.copy(self, obj, usids_subs=usids_subs)
		SbmlObject.copy(self, obj)
		Variable.copy(self, obj, sids_subs=sids_subs, symbols_subs=symbols_subs)

		if obj.listOfReactants not in deletions and len(obj.listOfReactants) > 0:
			self.listOfReactants.copy(
				obj.listOfReactants,
				deletions=deletions,
				sids_subs=sids_subs,
				symbols_subs=symbols_subs
			)
		if obj.listOfModifiers not in deletions and len(obj.listOfModifiers) > 0:
			self.listOfModifiers.copy(
				obj.listOfModifiers,
				deletions=deletions,
				sids_subs=sids_subs,
				symbols_subs=symbols_subs
			)
		if obj.listOfProducts not in deletions and len(obj.listOfProducts) > 0:
			self.listOfProducts.copy(
				obj.listOfProducts,
				deletions=deletions,
				sids_subs=sids_subs,
				symbols_subs=symbols_subs
			)

		if obj.listOfLocalParameters not in deletions and len(obj.listOfLocalParameters) > 0:
			self.listOfLocalParameters.copy(
				obj.listOfLocalParameters,
				deletions=deletions,
				sids_subs=sids_subs,
				symbols_subs=symbols_subs,
				usids_subs=usids_subs
			)

		# Here we need to add the deleted local parameters to the symbols subs, because they were promoted
		for local_param in obj.listOfLocalParameters:
			if local_param in deletions and local_param.symbol.getSymbol() not in symbols_subs:
				new_symbol = symbols_subs[SympySymbol(local_param.getSbmlId())]
				symbols_subs.update({local_param.symbol.getSymbol(): new_symbol})

		if obj.kineticLaw is not None:
			self.kineticLaw = KineticLaw(self.__model, self)
			self.kineticLaw.copy(
				obj,
				symbols_subs=symbols_subs,
				conversion_factors=conversion_factors,
				extent_conversion=extent_conversion,
				time_conversion=time_conversion)

			self.value = MathFormula(self.__model)
			t_formula = self.kineticLaw.getDefinition(rawFormula=True).getInternalMathFormula()

			if extent_conversion is not None:
				t_formula /= extent_conversion.getInternalMathFormula()

			if time_conversion is not None:
				t_formula *= time_conversion.getInternalMathFormula()

			self.value.setInternalMathFormula(t_formula)
			self.constant = obj.constant

		self.reversible = obj.reversible
		self.fast = obj.fast

	def readSbml(self, reaction, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads an sbml reaction from sbml model """

		if sbml_level < 3:
			self.setUnits(self.__model.substanceUnits)
		else:
			self.setUnits(self.__model.extentUnits)

		SbmlObject.readSbml(self, reaction, sbml_level, sbml_version)

		if reaction.getListOfReactants():
			self.listOfReactants.readSbml(reaction.getListOfReactants(), sbml_level, sbml_version)

		if reaction.getListOfProducts():
			self.listOfProducts.readSbml(reaction.getListOfProducts(), sbml_level, sbml_version)

		if reaction.getListOfModifiers():
			self.listOfModifiers.readSbml(reaction.getListOfModifiers(), sbml_level, sbml_version)

		if reaction.isSetReversible():
			self.reversible = reaction.getReversible()

		if reaction.isSetFast():
			self.fast = reaction.getFast()

		if reaction.getKineticLaw() is not None:

			t_params = reaction.getKineticLaw().getListOfParameters()
			self.listOfLocalParameters.readSbml(t_params,
												sbml_level, sbml_version)
			self.kineticLaw = KineticLaw(self.__model, self)
			self.kineticLaw.readSbml(reaction.getKineticLaw().getMath(),
										sbml_level, sbml_version)

		Variable.readSbml(self, reaction, sbml_level, sbml_version)


	def readSbmlVariable(self, variable, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		# variable id
		self.symbol.readSbml(variable.getId(), sbml_level, sbml_version)

		# self.isInitialized = True
		if self.kineticLaw is not None:
			self.value = self.kineticLaw.getDefinition(rawFormula=True)
			self.constant = False
		else:
			self.constant = True


	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes an sbml reaction to a sbml model """

		sbml_reaction = sbml_model.createReaction()

		Variable.writeSbml(self, sbml_reaction, sbml_level, sbml_version)
		SbmlObject.writeSbml(self, sbml_reaction, sbml_level, sbml_version)

		if (sbml_level < 3 and not self.reversible) or sbml_level == 3:
			sbml_reaction.setReversible(self.reversible)

		if (sbml_level < 3 and self.fast) or (sbml_level == 3 and sbml_version == 1):
			sbml_reaction.setFast(self.fast)
		else:
			sbml_reaction.setFast(False)

		if self.listOfModifiers:
			for modifier in self.listOfModifiers:
				sbml_modifier = sbml_reaction.createModifier()
				modifier.writeSbml(sbml_modifier, sbml_level, sbml_version)

		if self.listOfReactants:
			for reactant in self.listOfReactants:
				sbml_reactant = sbml_reaction.createReactant()
				reactant.writeSbml(sbml_reactant, sbml_level, sbml_version)

		if self.listOfProducts:
			for product in self.listOfProducts:
				sbml_product = sbml_reaction.createProduct()
				product.writeSbml(sbml_product, sbml_level, sbml_version)

		if self.kineticLaw is not None:
			kinetic_law = sbml_reaction.createKineticLaw()
			kinetic_law.setMath(self.kineticLaw.getDefinition(rawFormula=True).getSbmlMathFormula(sbml_level, sbml_version))
			self.listOfLocalParameters.writeSbml(kinetic_law, sbml_level, sbml_version)


	def writeSbmlVariable(self, variable, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes an sbml reaction to a sbml model """

		pass


	def __str__(self):

		return self.getReactionDescription() + " (" + str(self.kineticLaw.getDefinition().getInternalMathFormula() )+ ")"

	def setKineticLaw(self, reaction_type, reversible, parameters=None, math=None):

		self.reversible = reversible

		if reaction_type == KineticLaw.UNDEFINED and math is not None:
			self.kineticLaw = KineticLaw(self.__model, self)
			self.kineticLaw.setPrettyPrintMathFormula(math, forcedConcentration=True)

		elif parameters is not None:
			if reaction_type == KineticLaw.MASS_ACTION:
				self.kineticLaw = KineticLaw(self.__model, self)
				self.kineticLaw.setMassAction(parameters, reversible)

			elif reaction_type == KineticLaw.MICHAELIS:
				self.kineticLaw = KineticLaw(self.__model, self)
				self.kineticLaw.setMichaelis(parameters)

			elif reaction_type == KineticLaw.HILL:
				self.kineticLaw = KineticLaw(self.__model, self)
				self.kineticLaw.setHill(parameters)

		self.value = self.kineticLaw.getDefinition()
		self.constant = False

	def updateKineticLaw(self, kinetic_law):
		""" Update the kinetic law of the reaction """
		self.kineticLaw.getDefinition().setMathFormula(kinetic_law)


	def getKineticLaw(self, math_type=MathFormula.MATH_DEVINTERNAL, forcedConcentration=False):
		""" Returns the kinetic law in the specified format """
		return self.kineticLaw.getMathFormula(math_type, forcedConcentration)


	def getKineticLawDerivative(self, variable, math_type):
		""" Returns the kinetic law's derivative in the specified format """
		return self.kineticLaw.getDefinition().getMathFormulaDerivative(variable, math_type)


	def getReactionDescription(self):
		""" Returns the reaction description """
		lhs = ""
		if len(self.listOfReactants) + len(self.listOfModifiers) > 0:

			for i_reactant, reactant in enumerate(self.listOfReactants):
				if i_reactant > 0:
					lhs += " + "

				if not reactant.stoichiometry.isOne():
					lhs += "%g " % reactant.stoichiometry.getValueMathFormula()

				lhs += reactant.getSpecies().getNameOrSbmlId()

			if len(self.listOfModifiers) > 0:
				for i_modifier, modifier in enumerate(self.listOfModifiers):
					if i_modifier + len(self.listOfReactants) > 0:
						lhs += " + "

					if not modifier.stoichiometry.isOne():
						lhs += "%g " % modifier.stoichiometry.getValueMathFormula()

					lhs += modifier.getSpecies().getNameOrSbmlId()

		rhs = ""
		if len(self.listOfProducts) + len(self.listOfModifiers) > 0:

			for i_product, product in enumerate(self.listOfProducts):
				if i_product > 0:
					rhs += " + "

				if not product.stoichiometry.isOne():
					lhs += "%g " % product.stoichiometry.getValueMathFormula()

				rhs += product.getSpecies().getNameOrSbmlId()

			if len(self.listOfModifiers) > 0:
				for i_modifier, modifier in enumerate(self.listOfModifiers):
					if i_modifier + len(self.listOfProducts) > 0:
						rhs += " + "

					if not modifier.stoichiometry.isOne():
						lhs += "%g " % modifier.stoichiometry.getValueMathFormula()

					rhs += modifier.getSpecies().getNameOrSbmlId()

		if self.reversible == True:
			arrow = " <-> "
		else:
			arrow = " -> "

		return lhs + arrow + rhs

	def getReactionKineticLaw(self):

		if self.kineticLaw is not None:
			if self.kineticLaw.reactionType is None:
				self.kineticLaw.findKineticLaw()

			return self.kineticLaw.reactionTypes[self.kineticLaw.reactionType]


	def getReactionParameters(self):

		if self.kineticLaw is not None:
			if self.kineticLaw.reactionType is None:
				self.kineticLaw.findKineticLaw()
			return self.kineticLaw.getParameters()


	def getReactionType(self):
		""" Returns the reaction type """
		if self.kineticLaw is not None:
			if self.kineticLaw.reactionType is None:
				self.kineticLaw.findKineticLaw()
			return self.kineticLaw.reactionType

	def renameSbmlId(self, old_sbml_id, new_sbml_id):

		if self.kineticLaw is not None:
			self.kineticLaw.renameSbmlId(old_sbml_id, new_sbml_id)

		# if the stoichiometry is a variable,then it's in the list of variables
		# if it isn't, then there shouldn't be anything to rename

		for reactant in self.listOfReactants:
			reactant.renameSbmlId(old_sbml_id, new_sbml_id)

		for modifier in self.listOfModifiers:
			modifier.renameSbmlId(old_sbml_id, new_sbml_id)

		for product in self.listOfProducts:
			product.renameSbmlId(old_sbml_id, new_sbml_id)


	def getODE(self, species, rawFormula=False, symbols=False):

		ode = MathFormula.ZERO

		if self.listOfReactants:

			for reactant in self.listOfReactants:

				t_ode = MathFormula.ZERO

				if reactant.getSpecies() == species:

					if not symbols:
						t_ode -= self.kineticLaw.getDefinition(rawFormula=rawFormula).getInternalMathFormula()
					else:
						t_ode -= self.symbol.getInternalMathFormula()

					if not reactant.stoichiometry.isOne():
						t_ode *= reactant.stoichiometry.getInternalMathFormula()

				ode += t_ode


		if self.listOfProducts:
			for product in self.listOfProducts:
				t_ode = MathFormula.ZERO

				if product.getSpecies() == species:

					if not symbols:
						t_ode += self.kineticLaw.getDefinition(rawFormula=rawFormula).getInternalMathFormula()
					else:
						t_ode += self.symbol.getInternalMathFormula()

					if not product.stoichiometry.isOne():
						t_ode *= product.stoichiometry.getInternalMathFormula()

				ode += t_ode

		t_formula = MathFormula(self.__model)
		t_formula.setInternalMathFormula(ode)

		return t_formula

	def getStoichiometryMatrix(self, subs={}, including_fast_reactions=True, including_slow_reactions=True, include_variable_stoichiometry=False):

		front = zeros(self.__model.nbOdes, 1)

		if (
			(not self.hasVariableStoichiometry() or include_variable_stoichiometry) and
			(
				(self.fast and including_fast_reactions)
				or (not self.fast and including_slow_reactions)
			)
		):
			if self.listOfReactants:
				for reactant in self.listOfReactants:
					if not reactant.getSpecies().boundaryCondition:
						index = reactant.getSpecies().ind
						stoichiometry = unevaluatedSubs(
							reactant.stoichiometry.getDeveloppedInternalMathFormula(),
							subs
						)
						front[index] = -stoichiometry + front[index]

			if self.listOfProducts:
				for product in self.listOfProducts:
					if not product.getSpecies().boundaryCondition:
						index = product.getSpecies().ind
						stoichiometry = unevaluatedSubs(
							product.stoichiometry.getDeveloppedInternalMathFormula(),
							subs
						)
						front[index] = stoichiometry + front[index]


		if not self.kineticLaw.isReversible(self.kineticLaw.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula()):
			return front

		else:
			back = zeros(self.__model.nbOdes, 1)
			if (
				(not self.hasVariableStoichiometry() or include_variable_stoichiometry) and
				(
					(self.fast and including_fast_reactions)
					or (not self.fast and including_slow_reactions)
				)
			):
				if self.listOfReactants:
					for reactant in self.listOfReactants:
						if not reactant.getSpecies().boundaryCondition:
							index = reactant.getSpecies().ind
							stoichiometry = unevaluatedSubs(
								reactant.stoichiometry.getDeveloppedInternalMathFormula(),
								subs
							)
							back[index] = stoichiometry + back[index]

				if self.listOfProducts:
					for product in self.listOfProducts:
						if not product.getSpecies().boundaryCondition:
							index = product.getSpecies().ind
							stoichiometry = unevaluatedSubs(
								product.stoichiometry.getDeveloppedInternalMathFormula(),
								subs
							)
							back[index] = -stoichiometry + back[index]

			return front.row_join(back)

	def hasVariableStoichiometry(self):
		return self.listOfReactants.hasVariableStoichiometry() or self.listOfProducts.hasVariableStoichiometry()

	def isReactant(self, species):

		for reactant in self.listOfReactants:
			if reactant.getSpecies() == species:
				return True

		return False

	def isProduct(self, species):

		for product in self.listOfProducts:
			if product.getSpecies() == species:
				return True

		return False

	def containsVariable(self, variable):

		return (
			variable.symbol.getInternalMathFormula() in self.kineticLaw.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula().atoms()
		)

	def getByXPath(self, xpath):

		if len(xpath) == 0:
			return self

		if len(xpath) > 1:
			if xpath[0] in ["sbml:listOfParameters", "listOfParameters"]:
				return self.listOfLocalParameters.getByXPath(xpath[1:])

			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@value":
			return self.getValue()

		elif xpath[0] == "@name":
			return self.getName()

		elif xpath[0] == "@id":
			return self.getSbmlId()

	def getXPath(self, attribute=None):

		xpath = "sbml:reaction"
		if self.__model.sbmlLevel == 1:
			xpath += "[@name='%s']" % self.getSbmlId()
		else:
			xpath += "[@id='%s']" % self.getSbmlId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])

	def isValid(self):
		return self.kineticLaw.getDefinition() is not None

	def getCompartment(self):
		if len(self.listOfReactants) > 0:
			return self.listOfReactants[0].getSpecies().getCompartment()
		elif len(self.listOfProducts) > 0:
			return self.listOfProducts[0].getSpecies().getCompartment()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.LibSigNetSimException import LibSigNetSimException


class SedmlException(LibSigNetSimException):
	pass

class SedmlMathException(SedmlException):
	pass

class SedmlModelLanguageNotSupported(SedmlException):
	pass

class SedmlModelNotFound(SedmlException):
	pass

class SedmlFileNotFound(SedmlException):
	pass

class SedmlUnknownURI(SedmlException):
	pass

class SedmlUnknownXPATH(SedmlException):
	pass

class SedmlNotImplemented(SedmlException):
	pass

# One step simulations cannot be executed as a single task, they must be part of a repeated task
# At least, that's what I understand
class SedmlOneStepTaskException(SedmlException):
	pass

# Mixed subtasks are not implemented yet
class SedmlMixedSubtasks(SedmlNotImplemented):
	pass

class SedmlMultipleModels(SedmlNotImplemented):
	pass

class SedmlModelObjectNotFound(SedmlException):
	pass